# create_dat_file.sh: automatically generate .hex images for use with the 
# BP140 memory components of cycle model systems

if [ "$#" -ne 1 ]; then
	echo "Usage: ./create_dat_file.sh <path_to_axf>"
	exit 1
fi

target_application_path=$1
target_application_dir=$(dirname $target_application_path)
mydir="$(dirname $0)"

fromelf_exe=$(which fromelf)
if [ ! -x "$fromelf_exe" ]; then
	echo "fromelf executable not found. Plase make sure Arm Compiler 6 binaries are on your path."
	exit 1
fi

if [ ! -f "$target_application_path" ]; then
	echo "File $target_application_path not found."
	exit 1
fi

fromelf --bin --bincombined --output=$target_application_dir/image.bin $target_application_path
fromelf_success=$?
if [ "$fromelf_success" -ne "0" ]; then
	exit 1
fi

${mydir}/bin2vhx.pl  $target_application_dir/image.bin $target_application_dir/image.dat --width=64
if [ -f $target_application_dir/image.dat ]; then
	sed 's/^........//' $target_application_dir/image.dat > $target_application_dir/image_mem64_lo.hex
	sed 's/........$//' $target_application_dir/image.dat > $target_application_dir/image_mem64_hi.hex
else
	echo "Error! image.dat not created!"
	exit 1
fi

