;;-----------------------------------------------------------------------------
;; The confidential and proprietary information contained in this file may
;; only be used by a person authorised under and to the extent permitted
;; by a subsisting licensing agreement from ARM Limited.
;;
;;  (C) COPYRIGHT 1994-2008,2010-2012,2019 by Arm Limited (or its affiliates).
;;      ALL RIGHTS RESERVED
;;
;; This entire notice must be reproduced on all copies of this file
;; and copies of this file may only be made by a person if such person is
;; permitted to do so under the terms of a subsisting license agreement
;; from ARM Limited.
;;-----------------------------------------------------------------------------


        PRESERVE8
        AREA   CORTEXA, CODE, READONLY   ; name this block of code

		IF	:LNOT::DEF: HAS_FPU
		GBLL	HAS_FPU
HAS_FPU			SETL	{FALSE}
		ENDIF

		IF	:LNOT::DEF: HAS_L2CC
		GBLL	HAS_L2CC
HAS_L2CC		SETL	{FALSE}
		ENDIF
			
	EXPORT Reset_Handler

Reset_Handler

   
; Standard definitions of mode bits and interrupt (I & F) flags in PSRs

Mode_USR        EQU     0x10
Mode_FIQ        EQU     0x11
Mode_IRQ        EQU     0x12
Mode_SVC        EQU     0x13
Mode_ABT        EQU     0x17
Mode_UNDEF      EQU     0x1B
Mode_SYS        EQU     0x1F

I_Bit           EQU     0x80 ; when I bit is set, IRQ is disabled
F_Bit           EQU     0x40 ; when F bit is set, FIQ is disabled

;==================================================================
; Disable Cortex-A MMU if enabled
;==================================================================

        MRC     p15, 0, r0, c1, c0, 0       ; Read CP15 Control Regsiter into r0
        TST     r0, #0x1                    ; Is the MMU enabled?
        BICNE   r0, r0, #0x1                ; Clear bit 0
        MCRNE   p15, 0, r0, c1, c0, 0       ; Write value back

;==================================================================
; Initialise Supervisor Mode Stack
; Note stack must be 8 byte aligned.
;==================================================================
    ;; which cpu is this
    MRC p15,0,r0,c0,c0,5
    AND r0, r0, #0x3

    CMP r0, #0 
        BEQ SPCPU0    

    CMP r0, #1 
        BEQ SPCPU1    
        
    CMP r0, #2 
        BEQ SPCPU2    
        
    CMP r0, #3
        BEQ SPCPU3
        
SPCPU0
    IMPORT  ||Image$$STACK0$$ZI$$Limit|| ; Linker symbol from scatter file
    LDR     SP, =||Image$$STACK0$$ZI$$Limit||
    B      SPDONE
SPCPU1  
    IMPORT  ||Image$$STACK1$$ZI$$Limit|| ; Linker symbol from scatter file
    LDR     SP, =||Image$$STACK1$$ZI$$Limit||
    B      SPDONE
SPCPU2
    IMPORT  ||Image$$STACK2$$ZI$$Limit|| ; Linker symbol from scatter file
    LDR     SP, =||Image$$STACK2$$ZI$$Limit||
    B      SPDONE
SPCPU3
    IMPORT  ||Image$$STACK3$$ZI$$Limit|| ; Linker symbol from scatter file
    LDR     SP, =||Image$$STACK3$$ZI$$Limit||
SPDONE

;==================================================================
; TLB maintenace, Invalidate Data and Instruction TLB's
;==================================================================

        MOV    r0,#0
        MCR    p15, 0, r0, c8, c7, 0 ; Cortex-A I-TLB and D-TLB invalidation

;==================================================================
; Invalidate entire Branch predictor array
;==================================================================

        MOV    r0,#0
        MCR    p15, 0, r0, c7, c5, 6

;==================================================================
; Cache Invalidation code for Cortex-A
;==================================================================

        ; Invalidate L1 Instruction Cache

        MRC p15, 1, r0, c0, c0, 1   ; Read CLIDR
        TST r0, #0x3                ; Harvard Cache?
        MOV r0, #0
        MCRNE p15, 0, r0, c7, c5, 0 ; Invalidate Intruction Cache

        ; Invalidate Data/Unified Caches

        MRC p15, 1, r0, c0, c0, 1   ; Read CLIDR
        ANDS r3, r0, #&7000000
        MOV r3, r3, LSR #23         ; Total cache levels << 1

;===================================================================
; Cortex-A MMU Configuration
; Set translation table base
;===================================================================

        IMPORT ||Image$$TTB$$RW$$Base||  ; from scatter file.;

        ; Cortex-A supports two translation tables
        ; Configure translation table base (TTB) control register cp15,c2
        ; to a value of all zeros, indicates we are using TTB register 0.

        MOV     r0,#0x0
        MCR     p15, 0, r0, c2, c0, 2

        ; write the address of our page table base to TTB register 0.;
        ; We are setting to outer-noncachable [4:3] is zero

        LDR     r0,=||Image$$TTB$$RW$$Base||
        MCR     p15, 0, r0, c2, c0, 0

;===================================================================
; Setup domain control register - Enable all domains to client mode
;===================================================================

        MRC     p15, 0, r0, c3, c0, 0     ; Read Domain Access Control Register
        LDR     r0, =0x55555555           ; This initialize every domain entry to b01 (client)
        MCR     p15, 0, r0, c3, c0, 0     ; Write Domain Access Control Register

;==================================================================
; Enable access to NEON/VFP by enabling access to  Coprocessors 10 and 11.
; Enables Full Access .ie in both priv and non priv modes
;==================================================================

;1. Set bits [11:10] of the NSACR for access to CP10 and CP11 from both Secure and Non-secure states:
        MRC     p15, 0, r0, c1, c1, 2     ; read CP access register
        ORR     r0, r0, #2_11<<10         ; enable fpu
        MCR     p15, 0, r0, c1, c1, 2     ; write CP access register back

;2. Set the CPACR for access to CP10 and CP11:
        LDR r0, =(0xF << 20)
        MCR p15, 0, r0, c1, c0, 2

;==================================================================
; Switch on the VFP and Neon Hardware
;=================================================================

;3. Set the FPEXC EN bit to enable the FPU:
        MOV     r3, #0x40000000
        IF HAS_FPU = {TRUE}
        ;;; the following will cause an exception if no FPU is available
        FMXR    FPEXC, r3                 ; Write FPEXC register, EN bit set.
        VMOV    s0, r0                    ; Check to see if FPU is enabled
        ENDIF

;==================================================================
; Enable IRQ handling
;=================================================================
    ;; which cpu is this
    MRC p15,0,r0,c0,c0,5
    AND r0, r0, #0x3
    CMP r0, #0 
    BEQ IRQSPCPU0    
    CMP r0, #1 
    BEQ IRQSPCPU1    
    CMP r0, #2 
    BEQ IRQSPCPU2    
    CMP r0, #3 
    BEQ IRQSPCPU3    
IRQSPCPU0
    IMPORT  ||Image$$IRQ_STACK0$$ZI$$Limit|| ; Linker symbol from scatter file
    MSR	CPSR_c, #Mode_IRQ:OR:I_Bit:OR:F_Bit  ; Goto IRQ Mode, no interrupts
    LDR     SP, =||Image$$IRQ_STACK0$$ZI$$Limit|| ; Set up IRQ mode stack
	MSR	CPSR_c, #Mode_SVC:OR:F_Bit  ; Back to SVC mode with IRQ enabled
    B      IRQSPDONE
IRQSPCPU1
; CPU1 not doing interrupts
;    IMPORT  ||Image$$IRQ_STACK1$$ZI$$Limit|| ; Linker symbol from scatter file
;    MSR	CPSR_c, #Mode_IRQ:OR:I_Bit:OR:F_Bit  ; Goto IRQ Mode, no interrupts
;    LDR     SP, =||Image$$IRQ_STACK1$$ZI$$Limit|| ; Set up IRQ mode stack
;	MSR	CPSR_c, #Mode_SVC:OR:F_Bit  ; Back to SVC mode with IRQ enabled
;    B      IRQSPDONE
IRQSPCPU2
; CPU2 not doing interrupts
;    IMPORT  ||Image$$IRQ_STACK2$$ZI$$Limit|| ; Linker symbol from scatter file
;    MSR	CPSR_c, #Mode_IRQ:OR:I_Bit:OR:F_Bit  ; Goto IRQ Mode, no interrupts
;    LDR     SP, =||Image$$IRQ_STACK1$$ZI$$Limit|| ; Set up IRQ mode stack
;	MSR	CPSR_c, #Mode_SVC:OR:F_Bit  ; Back to SVC mode with IRQ enabled
;    B      IRQSPDONE
IRQSPCPU3
; CPU3 not doing interrupts
;    IMPORT  ||Image$$IRQ_STACK3$$ZI$$Limit|| ; Linker symbol from scatter file
;    MSR	CPSR_c, #Mode_IRQ:OR:I_Bit:OR:F_Bit  ; Goto IRQ Mode, no interrupts
;    LDR     SP, =||Image$$IRQ_STACK1$$ZI$$Limit|| ; Set up IRQ mode stack
;	MSR	CPSR_c, #Mode_SVC:OR:F_Bit  ; Back to SVC mode with IRQ enabled
;    B      IRQSPDONE
IRQSPDONE

;===================================================================
; Store address of branch to __main and enable MMU
;===================================================================

        IMPORT  __main                      ; before MMU enabled import label to __main
        LDR     r12,=__main                 ; save this in register for possible long jump

        MRC     p15, 0, r0, c1, c0, 0       ; read CP15 register 1 into r0
        ORR     r0, r0, #0x1                ; enable MMU before scatter loading
        MCR     p15, 0, r0, c1, c0, 0       ; write CP15 register 1

; Now the MMU is enabled, virtual to physical address translations will occur. This will affect the next
; instruction fetch.
;

;==================================================================
; Initialise Global Data
;==================================================================
    ;; which cpu is this
    IMPORT  ||Image$$GLOBAL$$RW$$Base|| ; Linker symbol from scatter file
    LDR r1, =||Image$$GLOBAL$$RW$$Base||
    MOV r3,#0x0
    MRC p15,0,r0,c0,c0,5
    AND r0, r0, #0x3
    CMP r0, #0 
    BEQ GLBCPU0    
    CMP r0, #1 
    BEQ GLBCPU1    
    CMP r0, #2 
    BEQ GLBCPU2    
    CMP r0, #3 
    BEQ GLBCPU3    
GLBCPU0
    STR     r3, [r1, #0x0]
    B   GLBDONE
GLBCPU1  
    STR     r3, [r1, #0x4]
    B   GLBDONE
GLBCPU2  
    STR     r3, [r1, #0x8]
    B   GLBDONE
GLBCPU3  
    STR     r3, [r1, #0xC]
    B   GLBDONE
GLBDONE

; The two instructions currently in the ARM pipeline will have been fetched before the MMU was enabled.
; This property is useful because the next two instructions are safe even if new instruction fetches fail -
; If this routine was mapped out of the new virtual memory map, the branch to __main would still succeed.

        BX      r12                         ; branch to __main  C library entry point

        LTORG

; Assembly to flush writes to memory by issuing a Data Memory Barrier instruction (DMB)
        EXPORT flushWrites        
flushWrites
        DMB
        BX	lr
                
        END                                 ; mark the end of this file

; End of File Init.s
