;;-----------------------------------------------------------------------------
;; The confidential and proprietary information contained in this file may
;; only be used by a person authorised under and to the extent permitted
;; by a subsisting licensing agreement from ARM Limited.
;;
;;   (C) COPYRIGHT 1994-2007,2010-2012,2019 by Arm Limited (or its affiliates).
;;       ALL RIGHTS RESERVED
;;
;; This entire notice must be reproduced on all copies of this file
;; and copies of this file may only be made by a person if such person is
;; permitted to do so under the terms of a subsisting license agreement
;; from ARM Limited.
;;-----------------------------------------------------------------------------

; =============================================================================
; User configurable L2-Cache architecture, user can modify the parameter values
; here to define his own L2-cache, re-build is needed to take effects.
; =============================================================================


; Include common define headers
#include "memory_map.h"



        AREA   |.text|, CODE, READONLY      ; name this block of code
        EXPORT l2cc_pl310_init

; ---------------------------------------------------------------------
; L2CC_WaySize : the way size for the L2-cache, the following value
;                is valid for this parameter configuration :
;                0x0 : reserved, internally mapped to 16KB;
;                0x1 : 16KB, default value;
;                0x2 : 32KB;
;                0x3 : 64KB;
;                0x4 : 128KB;
;                0x5 : 256KB;
;                0x6 : 512KB;
;                0x7 : reserved, internally mapped to 512KB;
; ---------------------------------------------------------------------

L2CC_WaySize    EQU     0x6

; ---------------------------------------------------------------------
; L2CC_WayNum : the number of ways in the L2-cache, the following value
;               value is valid for this parameter configuration :
;               0x0 : 8-way, this is the default;
;               0x1 : 16-way
; ---------------------------------------------------------------------

L2CC_WayNum     EQU     0x1

; ---------------------------------------------------------------------
; L2CC_TagWriteLatency  : cycles of latency for tag RAM read   :
; L2CC_TagReadLatency   : cycles of latency for tag RAM write  :
; L2CC_TagSetupLatency  : cycles of latency for tag RAM setup  :
; L2CC_DataReadLatency  : cycles of latency for data RAM read  :
; L2CC_DataWriteLatency : cycles of latency for data RAM write :
; L2CC_DataSetupLatency : cycles of latency for data RAM setup :
;
;   0x0 : 1 cycle of latency, no additional latency;
;   0x1 : 2 cycles of latency;
;   0x2 : 3 cycles of latency;
;   0x3 : 4 cycles of latency;
;   0x4 : 5 cycles of latency;
;   0x5 : 6 cycles of latency;
;   0x6 : 6 cycles of latency;
;   0x7 : 8 cycles of latency, this is default.
; ---------------------------------------------------------------------

L2CC_TagWriteLatency   EQU     0x2
L2CC_TagReadLatency    EQU     0x2
L2CC_TagSetupLatency   EQU     0x0

L2CC_DataWriteLatency  EQU     0x2
L2CC_DataReadLatency   EQU     0x2
L2CC_DataSetupLatency  EQU     0x0

; =============================================================================
; L2 Cache REGFILEBASE address - user can modify this value to define his own system
; =============================================================================

L2CC_RegBase    EQU       L2CCBase  ; L2CCBase from memory_map.h defines L2CC_RegBase        

; =============================================================================
; PL310 L2-Cache register offset declaration
; =============================================================================

L2CC_CacheID              EQU    L2CC_RegBase + 0x000
L2CC_CacheType            EQU    L2CC_RegBase + 0x004
L2CC_Ctrl                 EQU    L2CC_RegBase + 0x100
L2CC_AuxCtrl              EQU    L2CC_RegBase + 0x104
L2CC_TagLatency           EQU    L2CC_RegBase + 0x108
L2CC_DataLatency          EQU    L2CC_RegBase + 0x10c
L2CC_EvtCtrCtrl           EQU    L2CC_RegBase + 0x200
L2CC_EvtCtr1Cnfg          EQU    L2CC_RegBase + 0x204
L2CC_EvtCtr0Cnfg          EQU    L2CC_RegBase + 0x208
L2CC_EvtCtr1Val           EQU    L2CC_RegBase + 0x20c
L2CC_EvtCtr0Val           EQU    L2CC_RegBase + 0x210
L2CC_IntrMask             EQU    L2CC_RegBase + 0x214
L2CC_MaskIntrStatus       EQU    L2CC_RegBase + 0x218
L2CC_RawIntrStatus        EQU    L2CC_RegBase + 0x21c
L2CC_IntrClr              EQU    L2CC_RegBase + 0x220
L2CC_CacheSync            EQU    L2CC_RegBase + 0x730
L2CC_InvaLinePA           EQU    L2CC_RegBase + 0x770
L2CC_InvalByWay           EQU    L2CC_RegBase + 0x77c
L2CC_CleanLineByPA        EQU    L2CC_RegBase + 0x7b0
L2CC_CleanLineByIndexWay  EQU    L2CC_RegBase + 0x7b8
L2CC_CleanByWay           EQU    L2CC_RegBase + 0x7bc
L2CC_CleanInvalByPA       EQU    L2CC_RegBase + 0x7f0
L2CC_CleanInvalByIndexWay EQU    L2CC_RegBase + 0x7f8
L2CC_CleanInvalByWay      EQU    L2CC_RegBase + 0x7fc
L2CC_DataLockdown0ByWay   EQU    L2CC_RegBase + 0x900
L2CC_InstrLockdown0ByWay  EQU    L2CC_RegBase + 0x904
L2CC_DataLockdown1ByWay   EQU    L2CC_RegBase + 0x908
L2CC_InstrLockdown1ByWay  EQU    L2CC_RegBase + 0x90c
L2CC_DataLockdown2ByWay   EQU    L2CC_RegBase + 0x910
L2CC_InstrLockdown2ByWay  EQU    L2CC_RegBase + 0x914
L2CC_DataLockdown3ByWay   EQU    L2CC_RegBase + 0x918
L2CC_InstrLockdown3ByWay  EQU    L2CC_RegBase + 0x91c
L2CC_DataLockdown4ByWay   EQU    L2CC_RegBase + 0x920
L2CC_InstrLockdown4ByWay  EQU    L2CC_RegBase + 0x924
L2CC_DataLockdown5ByWay   EQU    L2CC_RegBase + 0x928
L2CC_InstrLockdown5ByWay  EQU    L2CC_RegBase + 0x92c
L2CC_DataLockdown6ByWay   EQU    L2CC_RegBase + 0x930
L2CC_InstrLockdown6ByWay  EQU    L2CC_RegBase + 0x934
L2CC_DataLockdown7ByWay   EQU    L2CC_RegBase + 0x938
L2CC_InstrLockdown7ByWay  EQU    L2CC_RegBase + 0x93c
L2CC_LockdownByWayEnable  EQU    L2CC_RegBase + 0x950
L2CC_UnlockAllLinesByWay  EQU    L2CC_RegBase + 0x954
L2CC_AddrFilterStart      EQU    L2CC_RegBase + 0xc00
L2CC_AddrFilterEnd        EQU    L2CC_RegBase + 0xc04



; =============================================================================
; Init and enable the L2-cache (L2CC/PL310)
; =============================================================================

l2cc_pl310_init    

    IF HAS_L2CC = {TRUE}

    ; =============================================================================
    ; Configure the L2-cache 
    ; =============================================================================

    ; -----------------------------------------------------------------------------
    ; check whether the L2-cache is enabled, if is, start to invalidate L2-cache,
    ; after that, configure the L2-cache registers.
    ; -----------------------------------------------------------------------------


        ; Only CPU0 to configure L2CC
        MRC p15,0,r0,c0,c0,5
        AND r0, r0, #3
        CMP    r0, #0            ; L2 cache to be done by CPU0 only
        BNE L2CC_DONE    

Check_L2Cache

        LDR    r0, =L2CC_Ctrl
        LDR    r1, [r0]
        AND    r1, r1, #0x1
        MOV    r2, #0x1
        CMP    r2, r1
        BNE    Config_L2Cache

Invalidate_L2Cache  ; start background cache invalidation process

        LDR    r3, =L2CC_InvalByWay
        LDR    r4, =0xffff
        MOV    r5, #0x0
        STR    r4, [r3]

Check_Invalidate  ; poll the register to see when the invalidation is complete, indicated by all bits cleared

        LDR    r4, [r3]
        CMP    r4, r5
        BNE    Check_Invalidate

Disable_L2Cache

        MOV    r1, #0x0
        STR    r1, [r0]

; -----------------------------------------------------------------------------
; config the L2-cache auxiliary control register
; -----------------------------------------------------------------------------

Config_L2Cache

        LDR    r3, =L2CC_WaySize
        LDR    r4, =L2CC_WayNum
        AND    r3, r3, #0x7
        AND    r4, r4, #0x1
        LDR    r0, =L2CC_AuxCtrl
        LDR    r1, [r0]
        BIC    r1, #0xf0000
        ORR    r1, r1, r4, LSL #0x10
        ORR    r1, r1, r3, LSL #0x11
        STR    r1, [r0]

; -----------------------------------------------------------------------------
; config the L2-cache tag memory latency register
; -----------------------------------------------------------------------------

        LDR    r3, =L2CC_TagWriteLatency
        LDR    r4, =L2CC_TagReadLatency
        LDR    r5, =L2CC_TagSetupLatency
        AND    r3, r3, #0x7
        AND    r4, r4, #0x7
        AND    r5, r5, #0x7
        ORR    r5, r5, r4, LSL #0x4
        ORR    r5, r5, r3, LSL #0x8
        LDR    r0, =L2CC_TagLatency
        STR    r5, [r0]

; -----------------------------------------------------------------------------
; config the L2-cache data memory latency register
; -----------------------------------------------------------------------------

        LDR    r3, =L2CC_DataWriteLatency
        LDR    r4, =L2CC_DataReadLatency
        LDR    r5, =L2CC_DataSetupLatency
        AND    r3, r3, #0x7
        AND    r4, r4, #0x7
        AND    r5, r5, #0x7
        ORR    r5, r5, r4, LSL #0x4
        ORR    r5, r5, r3, LSL #0x8
        LDR    r0, =L2CC_DataLatency
        STR    r5, [r0]


; -----------------------------------------------------------------------------
; Enable L2-cache
; -----------------------------------------------------------------------------

Enable_L2Cache

        LDR    r0, =L2CC_Ctrl
        MOV    r1, #0x1
        STR    r1, [r0]


L2CC_DONE


    ENDIF ; HAS_L2CC = {TRUE}

        BX lr

        END                                 ; mark the end of this file
; End of File InitL2CC.s
