/*
** Copyright (c) 2019 Arm Limited. All rights reserved.
*/

#include "pmu-and-marker.h"
#include <stdio.h>


#ifdef M7
#endif

#ifdef M0p
#endif

#ifdef M23
#endif

#ifdef M33
#endif

#if defined(R8) || defined(R52) || defined(A32) || defined(A5)
#include "v7_pmu.h"
#endif

#if defined(A35) || defined(A53)
#include "armv8_aarch64_pmu.h"
#endif


/* 
** Memory write to indicate the software region of interest is starting 
*/
void start_marker() 
{
    //ifdef for each core initalizing pmu counting
#if defined(M7) || defined (M33)
    CM_DEMCR |= CM_TRCENA_BIT;
    CM_DWT_LAR = 0xC5ACCE55;
    CM_DWT_CYCCNT = 0;
    CM_DWT_CONTROL |= CM_DWT_CYCCNTENA_BIT;
#endif

#if defined(M0p) || defined (M23)
    SysTick_RVR = SysTick_START;
    SysTick_CVR = 0;
    SysTick_CSR |=  (SysTick_Enable | SysTick_ClockSource);
#endif

#ifdef R52 
    enable_pmu_user_access();  // Allow access to PMU from User Mode

    enable_pmu();              // Enable the PMU

    pmn_config(0, 0x8);       // Configure counter 0 to count instructions retired
    pmn_config(1, 0x13);      // Configure counter 1 to count memory accesses
    pmn_config(2, 0x3);       // Configure counter 2 to count L1 dcache refill
    pmn_config(3, 0x4);       // Configure counter 3 to count L1 dcache accesses


    enable_pmn(0);             // Enable counter 0
    enable_pmn(1);             // Enable counter 1
    enable_pmn(2);             // Enable counter 2
    enable_pmn(3);             // Enable counter 3
    enable_ccnt();             // Enable CCNT

    reset_ccnt();              // Reset the CCNT (cycle counter)
    reset_pmn();               // Reset the configurable counters
#endif

#if defined(A5)
    enable_pmu_user_access();  // Allow access to PMU from User Mode

    enable_pmu();              // Enable the PMU

    pmn_config(0, 0x8);        // Configure counter 0 to count instructions retired
    pmn_config(1, 0x6);        // Configure counter 1 to count Loads

    enable_pmn(0);             // Enable counter 0
    enable_pmn(1);             // Enable counter 1
    enable_ccnt();             // Enable CCNT

    reset_ccnt();              // Reset the CCNT (cycle counter)
    reset_pmn();               // Reset the configurable counters
#endif


#if defined(A32) || defined(R8)
    enable_pmu_user_access();  // Allow access to PMU from User Mode

    enable_pmu();              // Enable the PMU

    pmn_config(0, 0x8);        // Configure counter 0 to count instructions retired
    pmn_config(1, 0x13);       // Configure counter 1 to count memory accesses
    pmn_config(2, 0x3);        // Configure counter 2 to count L1 dcache refill
    pmn_config(3, 0x4);        // Configure counter 3 to count L1 dcache accesses


    enable_pmn(0);             // Enable counter 0
    enable_pmn(1);             // Enable counter 1
    enable_pmn(2);             // Enable counter 2
    enable_pmn(3);             // Enable counter 3
    enable_ccnt();             // Enable CCNT

    reset_ccnt();              // Reset the CCNT (cycle counter)
    reset_pmn();               // Reset the configurable counters
#endif

#if defined(A53) || defined(A35)
    enablePMU();   // Enable PMU

    pmnConfig(0, 0x8, 0);     //Configure counter 0 to count intructions retired
    pmnConfig(1, 0x1, 0);     //Configure counter 1 to count ICache Miss
    pmnConfig(2, 0x4, 0);     //Configure counter 2 to count DCache Miss
    pmnConfig(3, 0x6, 0);     //Configure counter 3 to count Memory Reads

    //Enable the Programmable Counters
    enablePMN(0);
    enablePMN(1);
    enablePMN(2);
    enablePMN(3);
    enableCCNT();

    //Reset CCNT and the Counters
    resetCCNT();
    resetPMN();
#endif
}


/* 
** Memory write to indicate the software region of interest is ending 
*/
void stop_marker() 
{

  // In A5 DesignStart there's no requirement to implement Software Marker.
  // If implemented, the intent of it is to let trickbox know that PMU counting is on.
  //    SOFTWARE_MARK = 0;

#if defined(M7) || defined(M33)
    CM_DWT_CONTROL &= ~CM_DWT_CYCCNTENA_BIT;
    print_marker();
#endif

#if defined(M0p) || defined(M23)
    SysTick_CSR &= ~SysTick_Enable;

    print_marker();
#endif

#if defined(R8) || defined(R52) || defined(A32)
    disable_ccnt();            // Stop CCNT 
    disable_pmn(0);            // Stop counter 0 
    disable_pmn(1);            // Stop counter 1 
    disable_pmn(2);            // Stop counter 2 
    disable_pmn(3);            // Stop counter 3 
 
    print_marker();
#endif

#if defined(A5)
    disable_ccnt();            // Stop CCNT 
    disable_pmn(0);            // Stop counter 0 
    disable_pmn(1);            // Stop counter 1 
 
    print_marker();
#endif


#if defined(A53) || defined(A35)
    disableCCNT();
    disablePMN(0);
    disablePMN(1);
    disablePMN(2);
    disablePMN(3);

    print_marker();
#endif
}


/* 
** Print results of counters 
*/
void print_marker() 
{
    printf("\n______________________");
    printf("\nMarked software performance monitor results:\n\n");

#if defined(M7) || defined(M33)
    printf("CCNT = %u\n", CM_DWT_CYCCNT);
    printf("CPICNT = %u\n", CM_DWT_CPICNT);
    printf("EXCCNT  = %u\n", CM_DWT_EXCCNT);
    printf("SLEEPCNT = %u\n", CM_DWT_SLEEPCNT);
    printf("LSUCNT = %u\n", CM_DWT_LSUCNT);
    printf("FOLDCNT = %u\n", CM_DWT_FOLDCNT);
#endif

#if defined(M0p) || defined(M23)
    printf("CCNT = %u\n", (SysTick_START - SysTick_CVR));
    if (SysTick_CSR & 0x10000)
        printf("WARNING: counter has overflowed, more than 16,777,215 cycles");
#endif

#if defined(R52) || defined(R8) || defined(A32)
    float inst,cycl,cycl_inst;

    printf("Instructions Executed = %u\n",  read_pmn(0) );
    printf("Cycle Count (CCNT) = %u\n",     read_ccnt() );
    printf("Memory accesses = %u\n",        read_pmn(1) );
    printf("L1 Data cache refills = %u\n",  read_pmn(2) );
    printf("L1 Data cache accesses = %u\n", read_pmn(3) );

    inst=read_pmn(0);
    cycl=read_ccnt();
    cycl_inst=cycl/inst;

    printf("Average cycles per instruction = %f\n",cycl_inst);
#endif

#if defined(A5)
    float inst,cycl,cycl_inst;

    printf("Instructions Executed = %u\n",  read_pmn(0) );
    printf("Cycle Count (CCNT) = %u\n",     read_ccnt() );
    printf("Loads = %u\n",  read_pmn(1) );

    inst=read_pmn(0);
    cycl=read_ccnt();
    cycl_inst=cycl/inst;

    printf("Average cycles per instruction = %f\n",cycl_inst);
#endif


#if defined(A53) || defined(A35)
    float inst,cycl,cycl_inst;

    printf("Instructions Executed = %u\n",  readPMN(0) );
    printf("Cycle Count (CCNT) = %llu\n",     readCCNT() );
    printf("I Cache Misses = %u\n",        readPMN(1) );
    printf("L1 Data cache Misses = %u\n",  readPMN(2) );
    printf("Memory Reads = %u\n", readPMN(3) );

    inst=readPMN(0);
    cycl=readCCNT();
    cycl_inst=cycl/inst;

    printf("Average cycles per instruction = %f\n",cycl_inst);
#endif

    // End of printing
    printf("\nEnd of marked software performance monitor results.\n");
    printf("______________________\n");
}
