/*
** Copyright (c) 2019 Arm Limited. All rights reserved.
*/

#include <stdio.h>
#include <rt_misc.h>
#include "memory_map.h"	// For definition of Serial_Base
#include "serial.h"     // for simulation_exit

// Heap base from scatter file
extern int Image$$HEAP0$$ZI$$Base;
extern int Image$$HEAP1$$ZI$$Base;
extern int Image$$HEAP2$$ZI$$Base;
extern int Image$$HEAP3$$ZI$$Base;

// Initialisation function defined in InitCache.s
extern void cache_init(void);

// Initialisation function defined in InitL2CC.s
extern void l2cc_pl310_init(void);

// The application's Main function
extern int $Super$$main(char argc, char * argv[]);

extern int getCpuNum(void);
void record_boot(void);

/////////////////////////////////////////////////////////////////////////////
//
// This function is called at the end of the C library initialisation and
// before main. Its purpose is to do any further initialisation before the
// application start.
//
/////////////////////////////////////////////////////////////////////////////

void $Sub$$main(char argc, char * argv[])
{
  cache_init();               // does some extra setup work setting up caches
#ifdef HAS_L2CC  
  l2cc_pl310_init();
#endif  
  $Super$$main(argc, argv);   // calls the original function
    
  (void) simulation_exit();   // Stops simulation by writing a char of '4' to the trickbox
}

/////////////////////////////////////////////////////////////////////////////
//
// This function re-implements the C Library semihosted function. The stack 
// pointer has aready been set and is passed back to the funtion, The base of
// the heap is set from the scatter file.
//
/////////////////////////////////////////////////////////////////////////////

__value_in_regs struct __initial_stackheap __user_initial_stackheap(
        unsigned R0, unsigned SP, unsigned R2, unsigned SL)
{
    struct __initial_stackheap config;
    int cpuNum = getCpuNum();

	if (cpuNum == 0)
	{
		config.heap_base = (unsigned int)&Image$$HEAP0$$ZI$$Base; // placed by scatterfile
		config.stack_base = SP;   // inherit SP from the execution environment
	}
	else if (cpuNum == 1)
	{  
		config.heap_base = (unsigned int)&Image$$HEAP1$$ZI$$Base; // placed by scatterfile
		config.stack_base = SP;   // inherit SP from the execution environment
	}
	else if (cpuNum == 2)
	{  
		config.heap_base = (unsigned int)&Image$$HEAP2$$ZI$$Base; // placed by scatterfile
		config.stack_base = SP;   // inherit SP from the execution environment
	}
	else if (cpuNum == 3)
	{  
		config.heap_base = (unsigned int)&Image$$HEAP3$$ZI$$Base; // placed by scatterfile
		config.stack_base = SP;   // inherit SP from the execution environment
	}
	else 
	{
		while (1){} // Not CPU 0 or 1 so loop forever
	}
    return config;
}

// End of File retarget.c
