/*!
 *  \file   IrisCommon.h
 *  \brief  Declarations and definitions used by all Iris header files.
 *  \date   Copyright ARM Limited 2015 All Rights Reserved.
 *
 */

#ifndef ARM_INCLUDE_IrisCommon_h
#define ARM_INCLUDE_IrisCommon_h

#include "iris/detail/IrisSupportLibRevision.h"

#define IRIS_INNER_NAMESPACE IRIS_INNER_NAMESPACE_MAKE(IRIS_SUPPORT_LIB_REVISION,IRIS_SUPPORT_LIB_FORK)
#define IRIS_INNER_NAMESPACE_MAKE(revision,fork) IRIS_INNER_NAMESPACE_MAKE2(revision,fork)
#define IRIS_INNER_NAMESPACE_MAKE2(revision,fork) r##revision##fork

/// Start of 'iris' namespace.
/// Think of this as 'namespace iris {'.
/// This also starts the inner hidden revision namespace.
#define NAMESPACE_IRIS_START namespace iris { namespace IRIS_INNER_NAMESPACE {


/// End of 'iris' namespace in headers.
/// Think of this as the '}' matching NAMESPACE_IRIS_START.
/// This lifts the inner hidden revision namespace into 'iris'.
#define NAMESPACE_IRIS_END } using namespace IRIS_INNER_NAMESPACE; }

/// "fallthrough" attribute.
#if __cplusplus >= 201703L
// This is part of the language in C++17
#  define IRIS_FALLTHROUGH [[fallthrough]]
#else
#  if defined(__clang__)
#    if __has_cpp_attribute(clang::fallthrough)
#      define IRIS_FALLTHROUGH [[clang::fallthrough]]
#    endif
#  elif defined(__GNUC__)
#    if ((__GNUC__ > 7) || ((__GNUC__ == 7) && (__GNUC_MINOR__ >= 1)))
#      define IRIS_FALLTHROUGH [[gnu::fallthrough]]
#    endif
#  endif
#  if !defined(IRIS_FALLTHROUGH)
#    define IRIS_FALLTHROUGH
#  endif
#endif

/// "override" attribute.
#if __cplusplus >= 201103L
// This is part of the language in C++11
#  define IRIS_OVERRIDE override
#else
#  define IRIS_OVERRIDE
#endif

#include <stdint.h>


#endif // #ifndef ARM_INCLUDE_IrisCommon_h
