/*!
 *  \file  IrisDSO.h
 *  \brief Load shared libraries.
 *  \date  Copyright ARM Limited 2017. All rights reserved.
 */

#ifndef ARM_INCLUDE_IrisDSO_h
#define ARM_INCLUDE_IrisDSO_h

#include "IrisCommon.h"

#include <string>

NAMESPACE_IRIS_START

/// Load a DSO (a.k.a DLL, Shared Library) and read symbols from it
class IrisDSO
{
private:
    class Handle;

    Handle *handle;

public:
    IrisDSO(const std::string &dso_name);

    ~IrisDSO();

    /// Look up a symbol
    void *lookupSymbol(const std::string &symbol_name);

    /// Convenience function to lookup a symbol and cast it to the appropriate type.
    /// The output is via an argument passed by reference so that we can use
    /// template argument deduction to fill in the type for us.
    template <typename T>
    void lookupSymbol(const std::string &name, T& out)
    {
        out = reinterpret_cast<T>(lookupSymbol(name));
    }
};

NAMESPACE_IRIS_END

#endif // ARM_INCLUDE_IrisDSO_h
