/*!
 *  \file   IrisErrorCode.h
 *  \brief  Iris error codes.
 *  \date   Copyright ARM Limited 2014 All Rights Reserved.
 *
 *  Iris error codes as enum and string/value pairs.
 */

#ifndef ARM_INCLUDE_IrisErrorCode_h
#define ARM_INCLUDE_IrisErrorCode_h

#include "iris/detail/IrisCommon.h"

#include <string>

NAMESPACE_IRIS_START


    /// Define enum values.
    enum IrisErrorCode
    {
#define declare_enum(name, value) name = value,
#include "IrisErrorCodeList.h"
#undef declare_enum
    };

    /// Get 'E_...' string for enum value.
    /// Returned pointer is valid indefinitely and must not be freed.
    /// Returns "(unknown error code)" for invalid error codes.
    const char *irisErrorCodeCStr(IrisErrorCode errorCode);

    /// Same as irisErrorCodeCStr() but invalid error codes return
    /// "(unknown error code 0x<invalid_code>)".
    std::string irisErrorCodeString(IrisErrorCode errorCode);


NAMESPACE_IRIS_END

#endif // #ifndef ARM_INCLUDE_IrisErrorCode_h

