/*!
 *  \file   IrisErrorCodeList.h
 *  \brief  Iris error code list. This is not a normal header file and must not be included.
 *  \date   Copyright ARM Limited 2014 All Rights Reserved.
 *
 *  Iris error code list. This is not a normal header file and must not be included.
 *  This file is included several times by IrisErrorCode.h and *.cpp to provide an
 *  enum with introspection.
 */

// Do not add include guards. This file is included several times with different definitions
// for declare_enum.

// Documented error codes exposed by Iris functions.
declare_enum(E_u64json_encoding_error, -0xb0)
declare_enum(E_malformatted_request, -0xb1)
declare_enum(E_malformatted_response, -0xb2)
declare_enum(E_ok, 0x0000)
declare_enum(E_no_response_yet, 0x0001)
declare_enum(E_unknown_instance_id, 0xE100)
declare_enum(E_unknown_resource_id, 0xE101)
declare_enum(E_data_size_error, 0xE102)
declare_enum(E_unknown_memory_space_id, 0xE103)
declare_enum(E_unknown_event_source, 0xE104)
declare_enum(E_unknown_event_field, 0xE105)
declare_enum(E_unknown_event_stream_id, 0xE106)
declare_enum(E_unknown_disassembly_mode, 0xE107)
declare_enum(E_unknown_callback_instance_id, 0xE108)
declare_enum(E_syncEc_mode_not_supported, 0xE109)
declare_enum(E_counter_mode_not_supported, 0xE10A)
declare_enum(E_unknown_file_descriptor, 0xE10B)
declare_enum(E_invalid_context, 0xE10C)
declare_enum(E_unknown_sync_level, 0xE10D)
declare_enum(E_invalid_instName, 0xE10E)
declare_enum(E_instance_already_registered, 0xE10F)
declare_enum(E_function_not_supported_by_instance, 0xE110)
declare_enum(E_unsupported_argument_name, 0xE111)
declare_enum(E_unsupported_argument_value, 0xE112)
declare_enum(E_address_out_of_range, 0xE113)
declare_enum(E_unknown_resource_group, 0xE114)
declare_enum(E_unknown_event_source_id, 0xE115)
declare_enum(E_io_error, 0xE116)
declare_enum(E_unknown_image_format, 0xE117)
declare_enum(E_image_format_error, 0xE118)
declare_enum(E_error_opening_file, 0xE119)
declare_enum(E_missing_mandatory_argument, 0xE11A)
declare_enum(E_not_supported_for_event_source, 0xE11B)
declare_enum(E_not_a_counter, 0xE11C)
declare_enum(E_unaligned_access, 0xE11D)
declare_enum(E_unsupported_attribute_name, 0xE11E)
declare_enum(E_unsupported_attribute_value, 0xE11F)
declare_enum(E_unsupported_attribute_combination, 0xE120)
declare_enum(E_unsupported_breakpoint_type, 0xE121)
declare_enum(E_unsupported_argument_combination, 0xE122)
declare_enum(E_invalid_rwMode, 0xE123)
declare_enum(E_unknown_breakpoint_id, 0xE124)
declare_enum(E_unsupported_translation, 0xE125)
declare_enum(E_unit_not_supported, 0xE126)
declare_enum(E_writing_init_time_parameter, 0xE128)
declare_enum(E_not_connected, 0xE129)
declare_enum(E_index_out_of_range, 0xE12A)
declare_enum(E_unknown_table_id, 0xE12B)
declare_enum(E_argument_type_mismatch, 0xE12C)
declare_enum(E_not_supported_while_instance_is_blocked, 0xE12D)
declare_enum(E_unsupported_mode, 0xE12E)
declare_enum(E_unknown_instance_name, 0xE12F)
declare_enum(E_connection_refused, 0xE130) // IrisTcpSocket 57648
declare_enum(E_timeout, 0xE131)            // IrisTcpSocket 57649
declare_enum(E_unknown_hostname, 0xE132)   // IrisTcpSocket 57650
declare_enum(E_socket_error, 0xE133)       // IrisTcpSocket 57651
declare_enum(E_thread_error, 0xE134)       // IrisThreadAbstractionDefault
declare_enum(E_not_compatible, 0xE135)     // IrisRpcAdapterTcp
declare_enum(E_already_connected, 0xE136)  // IrisRpcAdapterTcp, IrisTcpClient
declare_enum(E_error_reading_write_only_resource, 0xE137)  // Instance implementation of ResourceReadFunctor::read().
declare_enum(E_error_writing_read_only_resource, 0xE138)  // Instance implementation of ResourceWriteFunctor::write().
declare_enum(E_error_reading_resource, 0xE139)  // Instance implementation of ResourceReadFunctor::read().
declare_enum(E_error_writing_resource, 0xE13A)  // Instance implementation of ResourceWriteFunctor::write().
declare_enum(E_not_implemented, 0xE13B)
declare_enum(E_operation_interrupted, 0xE13C)   // An operation was interrupted by an end user.
declare_enum(E_error_setting_breakpoint, 0xE13D)
declare_enum(E_error_deleting_breakpoint, 0xE13E)
declare_enum(E_error_creating_event_stream, 0xE13F)
declare_enum(E_error_enabling_event_stream, 0xE140)
declare_enum(E_error_disabling_event_stream, 0xE141)
declare_enum(E_error_memory_abort, 0xE142)
declare_enum(E_approximation, 0xE143)
declare_enum(E_value_not_available, 0xE144)
declare_enum(E_no_marshalling_for_current_thread, 0xE145)
declare_enum(E_server_already_running, 0xE146)
declare_enum(E_server_not_running, 0xE147)
declare_enum(E_error_loading_dso, 0xE148)
declare_enum(E_invalid_plugin, 0xE149)
declare_enum(E_plugin_already_loaded, 0xE14A)
declare_enum(E_error_writing_read_only_cell, 0xE14B)
declare_enum(E_invalid_breakpoint_condition, 0xE14C)
declare_enum(E_full_reset_not_supported, 0xE14D)
declare_enum(E_simulation_running, 0xE14E)

// These errors are not documented and are just used internally by low level functions.
declare_enum(E_internal_error, 0xEF00)
declare_enum(E_type_mismatch, 0xEF10)
declare_enum(E_unknown_member, 0xEF11)
declare_enum(E_json_parse_error, 0xEF12)
declare_enum(E_invalid_numeric_value, 0xEF13)


