/*!
 *  \file   IrisFunctionInfo.h
 *  \brief  FunctionInfo strings for all Iris functions.
 *  \date   Copyright ARM Limited 2017 - 2018. All Rights Reserved.
 *
 */

#ifndef ARM_INCLUDE_IrisFunctionInfo_h
#define ARM_INCLUDE_IrisFunctionInfo_h

#include "iris/detail/IrisCommon.h"

NAMESPACE_IRIS_START

namespace function_info
{

/// FunctionInfo for breakpoint_delete()
extern const char *breakpoint_delete;

/// FunctionInfo for breakpoint_getAdditionalConditions()
extern const char *breakpoint_getAdditionalConditions;

/// FunctionInfo for breakpoint_getList()
extern const char *breakpoint_getList;

/// FunctionInfo for breakpoint_set()
extern const char *breakpoint_set;

/// FunctionInfo for debuggableState_getAcknowledge()
extern const char *debuggableState_getAcknowledge;

/// FunctionInfo for debuggableState_setRequest()
extern const char *debuggableState_setRequest;

/// FunctionInfo for disassembler_disassembleOpcode()
extern const char *disassembler_disassembleOpcode;

/// FunctionInfo for disassembler_getCurrentMode()
extern const char *disassembler_getCurrentMode;

/// FunctionInfo for disassembler_getDisassembly()
extern const char *disassembler_getDisassembly;

/// FunctionInfo for disassembler_getModes()
extern const char *disassembler_getModes;

/// FunctionInfo for ec_FOO()
extern const char *ec_FOO;

/// FunctionInfo for eventRingBuffer_clear()
extern const char *eventRingBuffer_clear;

/// FunctionInfo for eventRingBuffer_init()
extern const char *eventRingBuffer_init;

/// FunctionInfo for eventRingBuffer_putBufferedEvents()
extern const char *eventRingBuffer_putBufferedEvents;

/// FunctionInfo for eventRingBuffer_read()
extern const char *eventRingBuffer_read;

/// FunctionInfo for eventStream_create()
extern const char *eventStream_create;

/// FunctionInfo for eventStream_destroy()
extern const char *eventStream_destroy;

/// FunctionInfo for eventStream_disable()
extern const char *eventStream_disable;

/// FunctionInfo for eventStream_enable()
extern const char *eventStream_enable;

/// FunctionInfo for eventStream_getCounter()
extern const char *eventStream_getCounter;

/// FunctionInfo for eventStream_getState()
extern const char *eventStream_getState;

/// FunctionInfo for eventStream_setTraceRanges()
extern const char *eventStream_setTraceRanges;

/// FunctionInfo for event_getEventSource()
extern const char *event_getEventSource;

/// FunctionInfo for event_getEventSources()
extern const char *event_getEventSources;

/// FunctionInfo for image_clearMetaInfoList()
extern const char *image_clearMetaInfoList;

/// FunctionInfo for image_getMetaInfoList()
extern const char *image_getMetaInfoList;

/// FunctionInfo for image_loadData()
extern const char *image_loadData;

/// FunctionInfo for image_loadDataPull()
extern const char *image_loadDataPull;

/// FunctionInfo for image_loadDataRead()
extern const char *image_loadDataRead;

/// FunctionInfo for image_loadFile()
extern const char *image_loadFile;

/// FunctionInfo for instanceRegistry_getInstanceInfoByInstId()
extern const char *instanceRegistry_getInstanceInfoByInstId;

/// FunctionInfo for instanceRegistry_getInstanceInfoByName()
extern const char *instanceRegistry_getInstanceInfoByName;

/// FunctionInfo for instanceRegistry_getList()
extern const char *instanceRegistry_getList;

/// FunctionInfo for instanceRegistry_registerInstance()
extern const char *instanceRegistry_registerInstance;

/// FunctionInfo for instanceRegistry_unregisterInstance()
extern const char *instanceRegistry_unregisterInstance;

/// FunctionInfo for instance_checkFunctionSupport()
extern const char *instance_checkFunctionSupport;

/// FunctionInfo for instance_getFunctionInfo()
extern const char *instance_getFunctionInfo;

/// FunctionInfo for instance_getProperties()
extern const char *instance_getProperties;

/// FunctionInfo for instance_ping()
extern const char *instance_ping;

/// FunctionInfo for memory_getMemorySpaces()
extern const char *memory_getMemorySpaces;

/// FunctionInfo for memory_getSidebandInfo()
extern const char *memory_getSidebandInfo;

/// FunctionInfo for memory_getUsefulAddressTranslations()
extern const char *memory_getUsefulAddressTranslations;

/// FunctionInfo for memory_read()
extern const char *memory_read;

/// FunctionInfo for memory_translateAddress()
extern const char *memory_translateAddress;

/// FunctionInfo for memory_write()
extern const char *memory_write;

/// FunctionInfo for perInstanceExecution_getState()
extern const char *perInstanceExecution_getState;

/// FunctionInfo for perInstanceExecution_getStateAll()
extern const char *perInstanceExecution_getStateAll;

/// FunctionInfo for perInstanceExecution_setState()
extern const char *perInstanceExecution_setState;

/// FunctionInfo for perInstanceExecution_setStateAll()
extern const char *perInstanceExecution_setStateAll;

/// FunctionInfo for plugin_getInstantiationParameterInfo()
extern const char *plugin_getInstantiationParameterInfo;

/// FunctionInfo for plugin_instantiate()
extern const char *plugin_instantiate;

/// FunctionInfo for plugin_load()
extern const char *plugin_load;

/// FunctionInfo for resource_getList()
extern const char *resource_getList;

/// FunctionInfo for resource_getListOfResourceGroups()
extern const char *resource_getListOfResourceGroups;

/// FunctionInfo for resource_read()
extern const char *resource_read;

/// FunctionInfo for resource_write()
extern const char *resource_write;

/// FunctionInfo for semihosting_notImplemented()
extern const char *semihosting_notImplemented;

/// FunctionInfo for semihosting_provideInputData()
extern const char *semihosting_provideInputData;

/// FunctionInfo for semihosting_return()
extern const char *semihosting_return;

/// FunctionInfo for service_connect()
extern const char *service_connect;

/// FunctionInfo for service_disconnect()
extern const char *service_disconnect;

/// FunctionInfo for simulationTime_get()
extern const char *simulationTime_get;

/// FunctionInfo for simulationTime_run()
extern const char *simulationTime_run;

/// FunctionInfo for simulationTime_runUntilDebuggableState()
extern const char *simulationTime_runUntilDebuggableState;

/// FunctionInfo for simulationTime_stop()
extern const char *simulationTime_stop;

/// FunctionInfo for simulation_getInstantiationParameterInfo()
extern const char *simulation_getInstantiationParameterInfo;

/// FunctionInfo for simulation_instantiate()
extern const char *simulation_instantiate;

/// FunctionInfo for simulation_requestShutdown()
extern const char *simulation_requestShutdown;

/// FunctionInfo for simulation_reset()
extern const char *simulation_reset;

/// FunctionInfo for simulation_setInstantiationParameterValues()
extern const char *simulation_setInstantiationParameterValues;

/// FunctionInfo for simulation_waitForInstantiation()
extern const char *simulation_waitForInstantiation;

/// FunctionInfo for step_getRemainingSteps()
extern const char *step_getRemainingSteps;

/// FunctionInfo for step_getStepCounterValue()
extern const char *step_getStepCounterValue;

/// FunctionInfo for step_setup()
extern const char *step_setup;

/// FunctionInfo for syncLevel_get()
extern const char *syncLevel_get;

/// FunctionInfo for syncLevel_release()
extern const char *syncLevel_release;

/// FunctionInfo for syncLevel_request()
extern const char *syncLevel_request;

/// FunctionInfo for table_getList()
extern const char *table_getList;

/// FunctionInfo for table_read()
extern const char *table_read;

/// FunctionInfo for table_write()
extern const char *table_write;

/// FunctionInfo for tcpServer_getPort()
extern const char *tcpServer_getPort;

/// FunctionInfo for tcpServer_start()
extern const char *tcpServer_start;

/// FunctionInfo for tcpServer_stop()
extern const char *tcpServer_stop;

} // namespace function_info

NAMESPACE_IRIS_END

#endif // ARM_INCLUDE_IrisFunctionInfo_h
