/*!
 *  \file   IrisJsonParser.h
 *  \brief  Convert JSON to U64JSON.
 *  \date   Copyright ARM Limited 2015 All Rights Reserved.
 *
 *  Convert JSON to U64JSON.
 *
 *  This JSON parser accepts a superset of all JSON values. Extensions over strict JSON
 *  are marked with 'Not strict JSON'. The extensions are in particular:
 *  - Strings can be quoted with single quotes (same semantics as double quotes).
 *  - Single quotes can be escaped in strings.
 *  - Hexadecimal integers, octal integers and hexadecimal floats are accepted.
 *  - Accept unary '+' before numbers.
 *  - \v can occur in strings
 */

#ifndef ARM_INCLUDE_IrisJsonParser_h
#define ARM_INCLUDE_IrisJsonParser_h

#include "iris/detail/IrisCommon.h"

#include <string>

NAMESPACE_IRIS_START

class IrisU64JsonWriter;

/// Convert JSON into U64JSON.
/// This parses exactly one JSON value and appends it to writer.
/// This is the only place in Iris where we parse JSON.
void convertJsonToU64Json(const std::string &json, IrisU64JsonWriter &writer);


NAMESPACE_IRIS_END

#endif // #ifndef ARM_INCLUDE_IrisJsonParser_h

