/*!
 *  \file   IrisJsonProducer.h
 *  \brief  Convert U64JSON to JSON.
 *  \date   Copyright ARM Limited 2015 All Rights Reserved.
 *
 *  Convert U64JSON to JSON.
 */

#ifndef ARM_INCLUDE_IrisJsonProducer_h
#define ARM_INCLUDE_IrisJsonProducer_h

#include "iris/detail/IrisCommon.h"
#include "iris/detail/IrisErrorCode.h"

#include <iostream>
#include <sstream>

NAMESPACE_IRIS_START

enum JsonFormat
{
    JsonNoSpacingFormat,
    JsonNoSpacingFormatIntegerObjects,
    JsonReadableFormat,
    IncorrectJsonSingleLineFormat,
    IncorrectJsonReadableFormat
};

/// Convert U64JSON to JSON.
/// This is used for interfacing purposes and also for display and debugging purposes.
/// Returns E_ok or E_u64json_encoding_error. In the error case an error message is printed to os (potentially preceeded by incomplete JSON text).
IrisErrorCode convertU64JsonToJson(JsonFormat format, const uint64_t *data, size_t size, std::ostream &os);

/// Convert U64JSON to JSON.
IrisErrorCode convertU64JsonToJson(JsonFormat format, const uint64_t *message, std::ostream &os);

/// Print Iris message in an invalid, yet more readable, format..
std::ostream& debugDumpMultiLine(const std::string &prefix, std::ostream& os, const uint64_t *message,
                                 bool printU64Json = false);


/// Convert Iris message to string (in IncorrectJsonSingleLineFormat, for debugging and error messages).
/// This is useful as argument to printf() like functions:
/// log.error("Invalid request:\n%s\n", messageToString(message));
std::string messageToString(const uint64_t *message);


NAMESPACE_IRIS_END

#endif // #ifndef ARM_INCLUDE_IrisJsonProducer_h

