/*!
 *  \file   IrisLogger.h
 *  \brief  Logging facilities for Iris framework.
 *  \date   Copyright ARM Limited 2015 All Rights Reserved.
 *
 *  Logging facilities for Iris framework.
 */

#ifndef ARM_INCLUDE_IrisLogger_h
#define ARM_INCLUDE_IrisLogger_h

#include "iris/detail/IrisCommon.h"
#include <cstdarg>
#include <cstdio>
#include <string>


NAMESPACE_IRIS_START

    
class IrisThreadAbstractionMutex;
   
class IrisLogger
{
public:
    /// If unsure, pass the instance name of the instance which owns this logger as logContext (directly or indirectly).
    IrisLogger(const std::string &logContext_ = std::string());

    /// Set log context.
    /// This string appears in the start of the log message to identify the instance which logs.
    /// Any instance name can be used for this.
    void setLogContext(const std::string &logContext_) 
    { 
        logContext = logContext_;
    }
    
    /// Log an error.    
    void error(const char *format, ...)
#ifdef __GNUC__
        __attribute__((format(printf, 2, 3)))
#endif
            ;

    /// Log a warning.
    void warning(const char *format, ...)
#ifdef __GNUC__
        __attribute__((format(printf, 2, 3)))
#endif
            ;

    /// Log informational messages.
    /// Please gate verbose messages with if (verbose) etc.
    void info(const char *format, ...)
#ifdef __GNUC__
        __attribute__((format(printf, 2, 3)))
#endif
            ;
    
    /// Free global mutex.
    static void cleanup();
    
private:
    // Private data.
    
    /// Log context, e.g. instance name.
    std::string logContext;
    
    /// Global mutex making sure multiple threads log sanely.
    static IrisThreadAbstractionMutex *mutex;
};

NAMESPACE_IRIS_END

#endif // #ifndef ARM_INCLUDE_IrisLogger_h

