/*!
 *  \file   IrisEventEmitter.h
 *  \brief  A utility class for emitting Iris Events
 *  \date   Copyright ARM Limited 2016 All Rights Reserved.
 *
 */

#ifndef ARM_INCLUDE_IrisRegisterEventEmitterBase_h
#define ARM_INCLUDE_IrisRegisterEventEmitterBase_h

#include "iris/detail/IrisCommon.h"
#include "iris/detail/IrisEventEmitterBase.h"
#include "iris/IrisInstanceBreakpoint.h"

NAMESPACE_IRIS_START

class IrisRegisterEventEmitterBase : public IrisEventEmitterBase
{
private:
    typedef std::vector<BreakpointInfo> BreakpointList;
    BreakpointList breakpoint_info;

    IrisInstanceBreakpoint *inst_breakpoint;

    static std::vector<uint64_t> marshalData(uint8_t value)  { return std::vector<uint64_t>(1, value); }
    static std::vector<uint64_t> marshalData(uint16_t value) { return std::vector<uint64_t>(1, value); }
    static std::vector<uint64_t> marshalData(uint32_t value) { return std::vector<uint64_t>(1, value); }
    static std::vector<uint64_t> marshalData(uint64_t value) { return std::vector<uint64_t>(1, value); }

    static std::vector<uint64_t> marshalData(int8_t value)  { return std::vector<uint64_t>(1, value); }
    static std::vector<uint64_t> marshalData(int16_t value) { return std::vector<uint64_t>(1, value); }
    static std::vector<uint64_t> marshalData(int32_t value) { return std::vector<uint64_t>(1, value); }
    static std::vector<uint64_t> marshalData(int64_t value) { return std::vector<uint64_t>(1, value); }

    static std::vector<uint64_t> marshalData(const std::vector<uint64_t> &value) { return value; }

    static std::vector<uint64_t> marshalData(const std::string &value);

protected:
    IrisRegisterEventEmitterBase(size_t num_fields_)
        : IrisEventEmitterBase(num_fields_)
        , inst_breakpoint(nullptr)
    {
    }

    template <typename REG_T>
    void checkBreakpointHit(uint64_t rscId, REG_T value, bool is_read_access)
    {
        BreakpointList::iterator it;
        for (it = breakpoint_info.begin(); it != breakpoint_info.end(); ++it)
        {
            const BreakpointInfo &info = *it;
            if (info.rscId == rscId)
            {
                // We assume there are only three possibilities for rwMode
                assert(info.rwMode == "rw" || info.rwMode == "r" || info.rwMode == "w");

                if (info.rwMode == "rw" || (is_read_access == (info.rwMode != "w")))
                {
                    // Hit!
                    std::vector<uint64_t> data = marshalData(value);
                    inst_breakpoint->notifyBreakpointHitRegister(info.bptId, IRIS_UINT64_MAX,
                                                                 IRIS_UINT64_MAX, IRIS_UINT64_MAX,
                                                                 is_read_access? "r" : "w",
                                                                 data);
                }
            }
        }
    }

public:
    void addBreakpoint(const BreakpointInfo &info);
    void deleteBreakpoint(uint64_t bptId);

    void setIrisInstanceBreakpoint(IrisInstanceBreakpoint *inst_breakpoint_)
    {
        inst_breakpoint = inst_breakpoint_;
    }
};

NAMESPACE_IRIS_END

#endif // ARM_INCLUDE_IrisRegisterEventEmitterBase_h
