/*!
 *  \file   IrisU64Json.h
 *  \brief  Utilities to convert data from and to U64JSON.
 *  \date   Copyright ARM Limited 2014 All Rights Reserved.
 *
 *  Utilities to convert data from and to U64JSON.
 */

#ifndef ARM_INCLUDE_IrisU64Json_h
#define ARM_INCLUDE_IrisU64Json_h

#include "iris/detail/IrisCommon.h"
#include "IrisInterface.h" // For uint64_t
#include <algorithm>
#include <vector>

NAMESPACE_IRIS_START


namespace u64json
{
    // U64JSON encoding constants.
    // Do not change. Changing these constants is not sufficient to keep the encoding consistent!
    static const uint64_t c_jsonrpc2 = 0x0020000000000000ull;
    static const uint64_t c_arrayu64 = 0x8000000000000000ull;
    static const uint64_t c_array    = 0xa000000000000000ull;
    static const uint64_t c_object   = 0xb000000000000000ull;
    static const uint64_t c_u64      = 0xc000000000000000ull;
    static const uint64_t c_s64      = 0xc100000000000000ull;
    static const uint64_t c_invalid  = 0xc9ffffffffffffffull;
    static const uint64_t c_double   = 0xca00000000000000ull;
    static const uint64_t c_string   = 0xcc00000000000000ull;
    static const uint64_t c_null     = 0xcd00000000000000ull;
    static const uint64_t c_false    = 0xce00000000000000ull;
    static const uint64_t c_true     = 0xcf00000000000000ull;
    static const uint64_t c_message  = 0xe000000000000000ull;
    static const uint64_t c_request  = 0xe000000000000000ull;
    static const uint64_t c_notification = 0xe100000000000000ull;
    static const uint64_t c_response = 0xe200000000000000ull;
    static const uint64_t c_msb4mask = 0xf000000000000000ull;
    static const uint64_t c_msb8mask = 0xff00000000000000ull;
    static const uint64_t c_msb16mask= 0xffff000000000000ull;
    static const uint64_t c_msg_max  = 0x0000ffffffffffffull;

    // Empty object container.
    extern const uint64_t empty_object[2];

} // namespace u64json

NAMESPACE_IRIS_END

#endif // #ifndef ARM_INCLUDE_IrisU64Json_h

