/**
 * \file    Framework/include/DWARF_Util.h
 * \date    Copyright (c) 2004 by ARM, Herzogenrath, Germany
 * \ingroup debug_info_database
 *
 * This file contains proprietary, unpublished source code
 * solely owned by ARM, Herzogenrath, Germany.
 * All rights reserved.
 */

#ifndef DWARF_UTIL_H
#define DWARF_UTIL_H

#include <cstdint>

#include <string>
#include <vector>

namespace DebugInfoDB
{
// read functions

    inline uint8_t DW_read1byte(const uint8_t *&cursor)
    {
        return *cursor++;
    }

    uint16_t DW_read2byte(const uint8_t *&cursor, bool isBigEndian);

    uint32_t DW_read4byte(const uint8_t *&cursor, bool isBigEndian);

    uint64_t DW_read8byte(const uint8_t *&cursor, bool isBigEndian);

    // read a 1,2,4 or 8 byte quantity
    uint64_t DW_readAddress(const uint8_t *&cursor, bool isBigEndian, uint32_t addressTypeSizeInBytes);

    // you really should provide an end pointer to avoid segfaults on corrupt input files
    uint64_t DW_readULEB128(const uint8_t *&cursor); // jo: dangerous function may segfault on end of buffer, do not use
    uint64_t DW_readULEB128(const uint8_t *&cursor, const uint8_t *end);

    // you really should provide an end pointer to avoid segfaults on corrupt input files
    int64_t DW_readSLEB128(const uint8_t *&cursor); // jo: dangerous function may segfault on end of buffer, do not use
    int64_t DW_readSLEB128(const uint8_t *&cursor, const uint8_t *end);

    // read a null terminated string
    const std::string DW_readString(const uint8_t *&cursor, const uint8_t *end);
}

#endif
