/**
 * \file    Framework/include/Dwarf2StringConstants.h
 * \date    Copyright (c) 2004 by ARM, Herzogenrath, Germany
 * \ingroup debug_info_database
 *
 * This file contains proprietary, unpublished source code
 * solely owned by ARM, Herzogenrath, Germany.
 * All rights reserved.
 */

#ifndef DWARF2STRINGCONSTANTS_H
#define DWARF2STRINGCONSTANTS_H

#include <cstdint>
#include "Dwarf2Constants.h"

namespace DebugInfoDB
{
    // these functions return a string for a numeric value
    // the returned pointer is valid at least until the next call to one of these functions
    const char *DW_TAG_GetString(DWARFTag tag);
    const char *DW_AT_GetString(DWARFAttribute at);
    const char *DW_FORM_GetString(DWARFFormat at);
    const char *DW_OP_GetString(DWARFOperation at);
    const char *DW_ATE_GetString(DWARFAttrEncoding at);
    const char *DW_ATE_GetStringReadElf(DWARFAttrEncoding at);
    const char *DW_ACCESS_GetString(DWARFAccessibility at);
    const char *DW_ACCESS_GetStringReadElf(DWARFAccessibility at);
    const char *DW_VIS_GetString(DWARFVisibility at);
    const char *DW_VIRTUALITY_GetString(DWARFVirtuality at);
    const char *DW_VIRTUALITY_GetStringReadElf(DWARFVirtuality at);
    const char *DW_LANG_GetString(DWARFSourceLanguage at);
    const char *DW_LANG_GetStringReadElf(DWARFSourceLanguage at);
    const char *DW_ID_GetString(DWARFIdentifierCase at);
    const char *DW_CC_GetString(DWARFCallingConvention at);
    const char *DW_INL_GetString(DWARFInline at);
    const char *DW_INL_GetStringReadElf(DWARFInline at);
    const char *DW_ORD_GetString(DWARFArrayOrdering at);
    const char *DW_DSC_GetString(DWARFDescriminant at);
    const char *DW_LNS_GetString(DWARFLineNumberOpcode at);
    const char *DW_LNE_GetString(DWARFLineNumberExtendedOpcode at);
    const char *DW_MACINFO_GetString(DWARFMacinfoType at);
    const char *DW_CFA_GetString(DWARFCallFrameInstruction at);
    const char *DW_CFA_RR_GetString(DWARFCallFrameInstruction at);
}

#endif
