/**
 * \file    Framework/include/FileInterface.h
 * \brief   File Interface implementation.
 * \date    Copyright (c) 2000-2011 by ARM
 * \ingroup file_interface
 *
 * This file contains proprietary, unpublished source code
 * solely owned by ARM
 * All rights reserved.
 */

#ifndef _FileInterface_h_
#define _FileInterface_h_

#include <stdio.h>
#include <string>
#include <stdlib.h>

class FileInterface
{
public:

#if defined(__GNUC__) || defined(__GNUG__) || defined(__clang__)

    typedef off_t                               FileOffsetType;

#elif defined(_MSC_VER)

    typedef __int64                             FileOffsetType;

#endif


    FileInterface(const char* filename);
    virtual ~FileInterface();

    virtual int fseek(FileOffsetType offset, int whence) = 0;
    virtual FileOffsetType ftell() = 0;
    virtual int feof() = 0;

    virtual size_t read(void *ptr, size_t count) = 0;

    virtual void rewind() = 0;
    char *fgets(char *s, int size);
    void getline(std::string& str);
    int fgetc();
    size_t fread(void *ptr, size_t size, size_t nmemb);

    char* filename;
};


class RawFileInterface: public FileInterface
{
public:

    typedef FileInterface::FileOffsetType       FileOffsetType;

    RawFileInterface(const char* filename);
    virtual ~RawFileInterface();

    virtual int fseek(FileOffsetType offset, int whence);
    virtual FileOffsetType ftell();
    virtual int feof();
    virtual size_t read(void *ptr, size_t count);

    virtual void rewind();

private:
    FILE* filehandle;
};


int fseek(FileInterface *stream, FileInterface::FileOffsetType offset, int whence);
FileInterface::FileOffsetType ftell(FileInterface *stream);
void rewind(FileInterface *stream);
int feof(FileInterface *stream);

size_t fread(void *ptr, size_t size, size_t nmemb, FileInterface *stream);
char *fgets(char *s, int size, FileInterface *stream);
FileInterface* getline(FileInterface *stream, std::string& str);
#endif
