/*
 * Int2String.h
 *
 * Copyright 2000-2003 ARM Limited.
 * Copyright 2013 ARM Limited.
 *
 * All rights reserved.
 */

/** \file
 * Macro expander header file.
 */

#ifndef Int2String_h
#define Int2String_h

#include "MxTypes.h"

#include <string>
#include <cstring>
#include <cstdlib>

/// Invalid integer value for number of digits for int2string
#define int_INVALID 0x7fffffff

/// Convert int to string
std::string int2string(int i, const char *format = "%d", int ndigits = int_INVALID);

/// Convert long to string
std::string long2string(long i, const char *format = "%d", int ndigits = int_INVALID);

/// Convert string to int
inline int string2int(const std::string& str, int base = 0) { return strtol(str.c_str(), 0, base); }

/// Convert string to bool
bool string2bool(const std::string& str, bool *error = 0);

/// Valid values for string2bool
extern const char *string2boolValidValues;

/// Convert bool to string
std::string bool2string (bool b);

/// Convert double to string
std::string double2string(double d, const char *format = "%g", int ndigits = int_INVALID);

/// Convert uint64_t into string
std::string uint64_t2string(uint64_t value, const char *format = "%" PRIu64, int ndigits = int_INVALID);

#ifdef WIN32
#define SIZE_T_FMT_SPEC "Iu"
#else
#define SIZE_T_FMT_SPEC "zu"
#endif

/// Convert size_t into string
std::string size_t2string(size_t value, const char *format = "%" SIZE_T_FMT_SPEC, int ndigits = int_INVALID);

// Convert string to MxU64
// (supports bin, hex, oct, dec, [mM], [kK] and [gG] suffixes (1024**n),
// positive and negative values, works like strtoull)
MxU64  ConstCharPtr2MxU64(const char   *str, const char **endPtr = 0, int base = 0);
inline uint64_t  ConstCharPtr2uint64_t(const char   *str, const char **endPtr = 0, int base = 0)
{
    return  (uint64_t)ConstCharPtr2MxU64(str,endPtr,base);
}

inline MxU64 string2MxU64(const std::string &str, const char **endPtr = 0, int base = 0) { return ConstCharPtr2MxU64(str.c_str(), endPtr, base); }
inline uint64_t string2uint64_t(const std::string &str, const char **endPtr = 0, int base = 0) { return ConstCharPtr2uint64_t(str.c_str(), endPtr, base); }

#endif


