/*!
 * \file    MxTypes.h
 * \brief   Types for components inherited from SoC Designer.
 * \date    Copyright 2000-2005 ARM Limited. All rights reserved.
 *          Copyright 2012-2013 ARM Limited. All rights reserved.
 *
 */


#ifndef MxTypes_h
#define MxTypes_h

/* Request xxx_C() macros */
#if !defined(__STDC_CONSTANT_MACROS)
    #define __STDC_CONSTANT_MACROS
#endif

/* Request xxx_MAX macros */
#if !defined(__STDC_LIMIT_MACROS)
    #define __STDC_LIMIT_MACROS
#endif

/* Request PRIxxx macro */
#if !defined(__STDC_FORMAT_MACROS)
    #define __STDC_FORMAT_MACROS
#endif

#include <inttypes.h> // use inttypes.h not cinttypes because this will in included into a Scanner.yy.c

/* Fix problems due to includes of Windows.h defining a macroised min and max
 * which conflict with std::min and std::max.
 */
#if defined(_WIN32) && !defined(NOMINMAX)
    #define NOMINMAX
#endif

// 8 bit types
typedef int8_t MxS8;
typedef uint8_t MxU8;

// 16 bit types
typedef int16_t MxS16;
typedef uint16_t MxU16;

// 32 bit types
typedef int32_t MxS32;
typedef uint32_t MxU32;

// 64 bit types
typedef int64_t MxS64;
typedef uint64_t MxU64;

enum MxStatus
{
    MX_STATUS_OK = 0,
    MX_STATUS_WAIT = 1,
    MX_STATUS_NOACCESS = 2,
    MX_STATUS_NOMEMORY = 3,
    MX_STATUS_ERROR = -1,
    MX_STATUS_NOTSUPPORTED = -2
};

#endif
