/**
 * \file    Framework/include/ObjectLoaderError.h
 * \brief   Loader error exception classes.
 * \date    Copyright (c) 2000-2003 by ARM, Herzogenrath, Germany
 * \ingroup lisa_simulator_generator
 *
 * This file contains proprietary, unpublished source code
 * solely owned by ARM, Herzogenrath, Germany.
 * All rights reserved.
 */

#ifndef _LoaderError_h_
#define _LoaderError_h_ 1

#include <stdio.h>
#include <string>
#include <exception>

// exception classes

// thrown by ObjectLoader::loadFile() and ObjectLoaderFormatSpecificLoaderXXX::loadFile()
// this class is used for actual loader error messages (like inconsistent file)
class ObjectLoaderError: public std::exception
{
public:
        ObjectLoaderError(const std::string& message_, int lineNumber = -1): message(message_)
        { if(lineNumber >= 0) { char buf[80]; sprintf(buf, " (line %d)", lineNumber); message += buf; }}

        virtual ~ObjectLoaderError() throw () {}

        std::string message;
        virtual const char* what() const throw() { return message.c_str (); }
};

// thrown by ObjectLoader::loadFile() and ObjectLoaderFormatSpecificLoaderXXX::loadFile()
// this class is used to signal that a specific loader can not load a specific file
// which is often not an error since it often occurs during the format probing
class ObjectLoaderErrorFormatNotRecognized: public ObjectLoaderError
{
public:
        ObjectLoaderErrorFormatNotRecognized(const std::string& message_): ObjectLoaderError(message_) {}
};

// thrown by ObjectLoader::loadFile() and ObjectLoaderFormatSpecificLoaderXXX::loadFile()
// this class is used to signal that the file could not be opened, probably because it does not exist
class ObjectLoaderErrorCouldNotOpenFile: public ObjectLoaderError
{
public:
        ObjectLoaderErrorCouldNotOpenFile(const std::string& message_): ObjectLoaderError(message_) {}
};

#endif

