/**
 * \file    Framework/include/ObjectLoaderMemoryConfInfo.h
 * \brief   ObjectLoader memory configuration information header file (interface specification)
 * \date    Copyright (c) 2000-2003 by ARM, Herzogenrath, Germany
 * \ingroup lisa_simulator_generator
 *
 * This file contains proprietary, unpublished source code
 * solely owned by ARM, Herzogenrath, Germany.
 * All rights reserved.
 */

#ifndef _ObjectLoaderMemoryConfInfo_h_
#define _ObjectLoaderMemoryConfInfo_h_ 1

#include <cstdint>

// static memory.conf information
struct ObjectLoaderAddressSpaceInfo
{
    uint32_t id;         // id != index
    uint32_t mauBytes;
    uint32_t addressInc;
    uint32_t addressBits;
    const char *name; // name of address space (e.g. 'program')
    bool isProgramMemory;
    bool isBigEndian;
};

struct ObjectLoaderSectionInfo
{
    const char *name; // name of section (e.g. '.text')
    uint32_t addressSpaceIndex;
    bool ignore;      // true == section is silently ignored during loading
    const char *access;
};

struct ObjectLoaderMemoryConfInfo
{
    uint32_t numAddressSpaces;
    const ObjectLoaderAddressSpaceInfo *addressSpace; // array
    uint32_t numSections;
    const ObjectLoaderSectionInfo *section; // array
    uint32_t programMemoryAddressSpaceIndex;
};

#endif

