/**
 * \file    Framework/include/ObjectLoaderRawInterface.h
 * \brief   Raw receiver header file (interface specification)
 * \date    Copyright (c) 2000-2003 by ARM, Herzogenrath, Germany
 * \ingroup lisa_simulator_generator
 *
 * This file contains proprietary, unpublished source code
 * solely owned by ARM, Herzogenrath, Germany.
 * All rights reserved.
 */

#ifndef _ObjectLoaderRawInterface_h_
#define _ObjectLoaderRawInterface_h_ 1

#include <cstdint>

#include "Elf.h"
#include "ObjectLoaderAddress.h"
#include "ObjectLoaderError.h"

// pure virtual interface class:
// this receiver class receives the raw data from the ObjectLoaderFormatSpecificLoader
// (only implemented by ObjectLoader)
class ObjectLoaderRawInterface
{
public:
    // virtual destructor
    virtual ~ObjectLoaderRawInterface() {}

    // put a piece of raw data
    virtual void putRawData(const char *sectionName, ObjectLoaderAddress startAddress, ObjectLoaderAddress size, const uint8_t *rawData,
                            uint32_t flags = SHF_EXECINSTR | SHF_ALLOC | SHF_WRITE, uint32_t type = SHT_PROGBITS) = 0;
    // put a symbol (label)
    virtual void putSymbol(const char *symbolName, ObjectLoaderAddress address, ObjectLoaderAddress size, const char *sectionName, uint32_t binding, uint32_t type) = 0;
    // set the start address of the program
    virtual void putProgramExecutionStartAddress(ObjectLoaderAddress address) = 0;
    // add a source line reference
    virtual void putSourceReference(ObjectLoaderAddress address, const char *sectionName, const char *fileName, int lineNumber) = 0;
    // transfer any other object file information
    // ('key' should specify the type of the information (e.g. 'weak_symbol'), 'key' should never be 0
    //  'sectionName' should be the section name associated with the info or 0 if not associated to any section
    //  'strValue' may be any string value or even 0,
    //  'numValue' may be any numeric value or even 0,
    //  'anyValue' may point to anything or may be even 0)
    // unused parameters should be se to 0
    virtual void putSpecificObjectInfo(const char *key, const char *sectionName, const char *strValue, ObjectLoaderAddress numValue, const void *anyValue) = 0;
};


#endif

