/*!
 *  \file   CheckpointInterface.h
 *  \brief  Component checkpoint (save and restore) API
 *  \date   Copyright ARM Limited 2013 All Rights Reserved.
 *
 */

#ifndef CHECKPOINT_INTERFACE_H
#define CHECKPOINT_INTERFACE_H

#include "eslapi/CAInterface.h"
#include "cp/StreamInterface.h"

#include <string>

namespace cp
{
    class CheckpointInterface : public eslapi::CAInterface
    {
    public:
        /**
         *  Checkpoint and return all the component's internal state.
         *  Given a pointer to the client's implementation of StreamInterface.
         *  \return true on success, false on failure.
         */
        virtual bool saveState(cp::StreamInterface *si) = 0;

        /**
         *  Restore a component to a previously checkpointed state.
         *  Must only be performed immediately after model instantiation/reset
         *  Given a pointer to the client's implementation of StreamInterface.
         *
         *  \return true on success, false on failure.
         */
        virtual bool restoreState(cp::StreamInterface *si) = 0;

        static eslapi::if_name_t IFNAME()     { return "CheckpointInterface0"; }
        static eslapi::if_rev_t  IFREVISION() { return 0; }
    };
}

#endif

// end of file CheckpointInterface.h
