/*!
 *  \file   CADIRemoteBroker.h
 *  \brief  Interface declaration for CADIRemoteBroker
 *  \date   Copyright (c) 2015 ARM. All Rights Reserved.
 */

#ifndef CADIRemoteBroker_H
#define CADIRemoteBroker_H

#include "eslapi/CADICommon.h"

namespace eslapi
{

    class CADIRemoteBroker: public CAInterface
    {
    public:

        /// Return the CAInterface name for this interface.
        static if_name_t IFNAME() { return "eslapi.CADIRemoteBroker2"; }

        /// Specify the current minor revision for this interface.
        static if_rev_t IFREVISION() { return 0; }

        /// Configures the CADIBroker simulation scan engine to connect to a specific remote host and port.
        /// The address must be null-terminated. The format for the address is [protocol://]host[:port].
        /// The protocol is optional; if given, it should be "cadiclient://".
        /// The host can be a hostname or IPv4 address.
        /// The port is optional; the default port is 31627.
        virtual CADIReturn_t SetServerAddress (const char address [CADI_NAME_SIZE]) = 0;

        /// Returns the currently configured remote server address that the
        /// CADIBroker simulation scan engine is using.
        virtual CADIReturn_t GetServerAddress (char address [CADI_NAME_SIZE]) = 0;
    };

}

#endif
