/*
 * SCX: scx_debug_disass_if.h - SystemC eXport support: Disassembly interface.
 *
 * Copyright 2016 ARM Limited.
 * All rights reserved.
 *
 * WARNING! This file is not part of a stable public API and
 *          is subject to change without notice.
 */

#ifndef ARM_SCX_DEBUG_DISASS_IF_H
#define ARM_SCX_DEBUG_DISASS_IF_H

/*!
 * \file        scx_debug_control_if.h
 * \brief       SystemC eXport support: Disassembly interface.
 * \date        Copyright 2016 ARM Limited. All rights reserved.
 *
 * This file specifies interfaces provided by models to debug target
 * for disassembly.
 */

/* Namespaces */

/*!
 * \brief   SystemC eXport support namespace.
 */
namespace scx {

/*!
 * \brief       Disassembly interface.
 *
 * This interface specifies operations provided by models to debug target
 * for disassembly.
 */
class scx_debug_disass_if {

    /* Construction */
    protected:

        /*!
         * \brief       Destructor.
         *
         * This version of the function does not allow destroying instance
         * through this interface.
         */
        virtual
        ~scx_debug_disass_if() {
        }

    /* Debug support */
    public:

        /*!
         * \brief       Return model current execution mode.
         *
         * \return      \c 0 when states are not supported, current execution
         *              number otherwise, as defined by scx_debug_target_if::add_mode().
         * \see         scx_debug_target_if::add_mode()
         */
        virtual uint32_t
        debug_get_curr_mode() = 0;

        /*!
         * The function enables standard type instruction disassembly.
         *
         * \param       space memory space identifier
         * \param       addr address of instruction to disassemble and address of
         *              next valid instruciton on return
         * \param       opcode opcode text for disassembled instruction
         * \param       disassembly text for disassembly
         * \param       mode contains execution mode, or \c 0 for current
         *              execution mode.
         * \return      \c true for sucessful disassembly, \c false otherwise.
         */
        virtual bool
        debug_disassemble(uint32_t space,
                          uint64_t & addr,
                          std::string & opcode,
                          std::string & disassembly,
                          uint32_t mode) = 0;
};

}   /* namespace scx */

#endif  /* defined(ARM_SCX_DEBUG_DISASS_IF_H) */
