/*
 * SCX: scx_debug_params_if.h - SystemC eXport support: Debug parameters resource interface.
 *
 * Copyright 2016 ARM Limited.
 * All rights reserved.
 *
 * WARNING! This file is not part of a stable public API and
 *          is subject to change without notice.
 */

#ifndef ARM_SCX_DEBUG_PARAMS_IF_H
#define ARM_SCX_DEBUG_PARAMS_IF_H

/*!
 * \file        scx_debug_params_if.h
 * \brief       SystemC eXport support: Debug parameters resource interface.
 * \date        Copyright 2016 ARM Limited. All rights reserved.
 *
 * This file specifies interfaces provided by models to debug target
 * for accessing their parameters resource.
 */

/* Namespaces */

/*!
 * \brief   SystemC eXport support namespace.
 */
namespace scx {

/* Datatypes */

/*!
 * \brief       Debug parameters resource interface.
 *
 * This interface specifies operations provided by models to debug target
 * for accessing their parameters resource.
 */
class scx_debug_params_if {

    /* Construction */
    protected:

        /*!
         * \brief   Destructor.
         *
         * This version of the function does not allow destroying instance
         * through this interface.
         */
        virtual
        ~scx_debug_params_if() {
        }

    /* Debug support */
    public:

        /*!
         * \brief   Get a parameter value.
         *
         * This function performs a read from a model parameter resource,
         * but without any delays, waits, event notifications, or side effects.
         * In other words, this function shall be non-intrusive.
         *
         * \param   name parameter name
         * \param   value reference to \c std::string parameter value (own by debug target).
         * \return  \c true when parameter successfully read, \c false otherwise
         * \see     scx_debug_target_if::add_param()
         */
        virtual bool
        debug_param_get(const std::string & name, std::string & value) = 0;

        /*!
         * \brief   Set a parameter value.
         *
         * This function performs a write to a model parameter resource,
         * but without any delays, waits, event notifications, or side effects.
         * In other words, this function shall be non-intrusive.
         *
         * \param   name parameter name
         * \param   value reference to \c std::string parameter value (own by debug target).
         * \return  \c true when parameter successfully written, \c false otherwise
         * \see     scx_debug_target_if::add_param()
         */
        virtual bool
        debug_param_set(const std::string & name, const std::string & value) = 0;
};

}   /* namespace scx */

#endif  /* defined(ARM_SCX_DEBUG_PARAMS_IF_H) */

