/*
 * SCX: scx_iris.h - SystemC eXport support: Iris simulation debug.
 *
 * Note: Iris is in pre-alpha stage and not usable yet.
 *       Do not expose this header to customers in release packages.
 *
 * Copyright 2015 - 2017 ARM Limited.
 * All rights reserved.
 */

#ifndef ARM_SCX_IRIS_H
#define ARM_SCX_IRIS_H

#include <iris/detail/IrisInterface.h>

#include <string>
#include <cstdint>

namespace scx {

/*!
 * \brief   Constants indicating the default port range used for the Iris server.
 */
static const uint16_t DefaultIrisServerPortMin = 7100;
static const uint16_t DefaultIrisServerPortMax = 7163;

/*!
 * \brief   Specifies whether to start an Iris server. If a server is started it will
 *          listen on a port in the range [DefaultIrisServerPortMin, DefaultIrisServerPortMax]
 */
void scx_enable_iris_server(bool enable = true);

/*!
 * \brief   Set the range of ports to scan when starting an Iris server. The first
 *          available port found will be used.
 */
void scx_set_iris_server_port_range(uint16_t port_min, uint16_t port_max);

/*!
 * \brief   Set a specific port to use for the Iris server.
 */
inline void scx_set_iris_server_port(uint16_t port)
{
    scx_set_iris_server_port_range(port, port);
}

/*!
 * \brief   Specifies Iris log level
 */
void scx_enable_iris_log(unsigned level = 0);

/*!
 * \brief   Retrieve the IrisConnectionInterface for the simulation. This can be used to create and
 *          register IrisInstances.
 */
iris::IrisConnectionInterface *scx_get_iris_connection_interface();
}
#endif
