/*
 * SCX: scx_module_base.h - SystemC eXport support: definition of module base class.
 *
 * Copyright 2016 ARM Limited.
 * All rights reserved.
 */

#ifndef ARM_SCX_MODULE_BASE_H
#define ARM_SCX_MODULE_BASE_H

/*!
 * \file    scx_module_base.h
 * \brief   Exported module base class.
 * \date    Copyright 2016 ARM Limited. All rights reserved.
 *
 * Definition of module base class.
 */

/* Includes */
#include <string>

/* Namespaces */

/*!
 * \brief   SystemC eXport support namespace.
 */
namespace scx {

/* Datatypes */

/*!
 * \brief   SystemC eXport module base class.
 *
 * This class triggers SimulationEngine behaviours on SystemC simulation call-backs.
 * It is intended to be inherited by EVS and external SystemC modules.
 *
 * \see     scx_evs_base, scx_target_base
 */
class scx_module_base {

    /* Construction */
    protected:
        explicit scx_module_base(const std::string &);

    /* Simulation callbacks */
        void before_end_of_elaboration();
        void end_of_elaboration();
        void start_of_simulation();
        void end_of_simulation();

    /* Member Variable */
        std::string m_name;
};

/* Functions */

}   /* namespace scx */

#endif  /* defined(ARM_SCX_MODULE_BASE_H) */
