/*
 * SCX: scx_ver.h - SystemC eXport support: Version control.
 *
 * Copyright 2016 ARM Limited.
 * All rights reserved.
 */

#ifndef ARM_SCX_VER_H
#define ARM_SCX_VER_H

/*!
 * \file        scx_ver.h
 * \brief       Version control.
 * \date        Copyright 2016 ARM Limited. All rights reserved.
 *
 * SystemC eXport support version control.
 */

/* Includes */
#include <PVVersion.h>

/* Namespaces */

/*!
 * \brief       SystemC eXport support namespace.
 */
namespace scx {

/* Globals */

#define SCX_CONCAT_HELPER(l, r) SCX_CONCAT_HELPER2(l, r)
#define SCX_CONCAT_HELPER2(l, r) l ## _ ## r
#define SCX_API_VERSION_CLASS_HELPER(v, r, b) \
    SCX_CONCAT_HELPER(scx_api_version, SCX_CONCAT_HELPER(v, SCX_CONCAT_HELPER(r, b)))
#define SCX_API_VERSION_CLASS   \
    SCX_API_VERSION_CLASS_HELPER(SYSGEN_PVMODELLIB_MAJOR, SYSGEN_PVMODELLIB_MINOR, SYSGEN_PVMODELLIB_BUILD)

/*!
 * \brief       Help detect bad version-revision-build objects at link time.
 *
 * Each source file which includes this file for current SystemC eXport
 * framework version will have static instance of class scx_api_version_X_Y_Z
 * defined in it,
 * where X_Y_Z is defined based on framework version major, minor and build.
 * That object instance will cause its constructor to be invoked.
 * If version of SystemC eXport framework being linked against does not
 * contain same constructor, linkage error will occur.
 *
 * \note        These checks can be disabled by defining
 *              \c FM_SCX_DISABLE_API_VERSION_CHECK macro before including SystemC
 *              eXport header files.
 */
struct SCX_API_VERSION_CLASS {
    SCX_API_VERSION_CLASS();
};

#if ! defined(FM_SCX_DISABLE_API_VERSION_CHECK)
/*!
 * \brief       Help detect bad version-revision-build objects at link time.
 *
 * Each source file which includes this file for current SystemC eXport
 * framework version will have static instance of class scx_api_version_X_Y_Z
 * defined in it,
 * where X_Y_Z is defined based on framework version major, minor and build.
 * That object instance will cause its constructor to be invoked.
 * If version of SystemC eXport framework being linked against does not
 * contain same constructor, linkage error will occur.
 *
 * \note        These checks can be disabled by defining
 *              \c FM_SCX_DISABLE_API_VERSION_CHECK macro before including SystemC
 *              eXport header files.
 */
static SCX_API_VERSION_CLASS scx_api_version_check;
#endif  /* defined(FM_SCX_DISABLE_API_VERSION_CHECK) */

}   /* namespace scx */

#endif  /* defined(ARM_SCX_VER_H) */

