/* -*- c++ -*- */
/* vim: syntax=cpp
 */
/*!
 *  \file   FsTypeConv.h
 *  \brief  Defines type conversion utilities.
 *  \date   Copyright ARM Limited 2013. All Rights Reserved.
 *
 *  This file is not part of a stable public API and it may
 *  change without notice.
 */

#ifndef INCLUDED_FSTYPECONV_H
#define INCLUDED_FSTYPECONV_H

namespace fs {

/*!
 * Type conversion or "type punning" function.
 * This function exists because of the C/C++ strict aliasing rules, and a union
 * allows to dereference the same memory location as different types (at least
 * this is standard C and supported by C++ and VC++ compilers).
 *
 * \param   v value of type \a T1 to convert
 * \return  representation of \a v reinterpreted as a value of type \a T2.
 *
 * \warning The representation of the return value might be a trap
 *          representation.
 */
template<typename T2, typename T1>
inline T2
type_conv(const T1 & v) {
    union {
        T1 a;
        T2 b;
    } tmp;

    tmp.a = v;
    return (tmp.b);
}

}   /* namespace fs */

#endif

/* eof FsTypeConv.h */
