/* -*- c++ -*- */
/* vim: syntax=cpp
 */
/*!
 *  \file   HPTime.h
 *  \brief  High Precision time reporting for Wall clocks
 *  \date   Copyright ARM Limited 2008 All Rights Reserved.
 *          Copyright ARM Limited 2008 All Rights Reserved.
 *
 *  This file is not part of a stable public API and it may
 *  change without notice.
 */

#ifndef INCLUDED_SG_HPTIME_H
#define INCLUDED_SG_HPTIME_H

#include <cstdint>

namespace sg
{

/*! Get the Highest Precision wall clock time that the OS can provide.
 *
 *  \remark value at process start is not defined and in particular not
 *          guaranteed to be zero.
 *  \remark Most Unix systems require the program to linked against the librt
 *          library to use these functions.
 *
 *  \param  tim buffer to accept time value
 *  \return 0 on success
 */
int GetHPWallTime(uint64_t* tim);

/*! Get the resolution of the timer provided by \ref GetHPWallTime
 *
 *  \remark Resolution is specified in nano seconds.
 *  \remark Most Unix systems require the program to linked against the librt
 *          library to use these functions.
 *
 *  \return 0 on success
 */
int GetHPWallTimeRes(uint64_t* res);

/*! Get the frequency of the timer provided by \ref GetHPWallTime
 *
 *  \remark Resolution is specified in counts per second
 *  \remark Most Unix systems require the program to linked against the librt
 *          library to use these functions.
 *
 *  \return 0 on success
 */

int GetHPWallTimeFreq(uint64_t* freq);

}

#endif

/* eof HPTime.h */
