/* -*- c++ -*- */
/* vim: syntax=cpp
 */
/*!
 *  \file   MSCFixes.h
 *  \brief  Portability fixes required for Microsoft C/C++
 *  \date   Copyright ARM Limited 2008 All Rights Reserved.
 *          Copyright ARM Limited 2017 All Rights Reserved.
 *
 *  Some versions of the Microsoft C/C++ compiler do not define many C99
 *  functions or defines them with different names.
 *
 *  This file aliases the functions to match gcc, where possible.
 *
 *  This file is not part of a stable public API and it may
 *  change without notice.
 *
 */

#ifndef INCLUDED_MSCFIXES_H
#define INCLUDED_MSCFIXES_H

#include <cstdio>

#ifdef _MSC_VER
    // Note: _snprintf may not null-terminate; use _snprintf_s instead
    #define snprintf(x, y, format, ...) _snprintf_s(x, y, _TRUNCATE, format, __VA_ARGS__)

    #define putenv _putenv

    #define setenv(x,y,z) _putenv_s(x,y)

    #define strtoll _strtoi64

    #define strtoull _strtoui64

    #define strdup _strdup

    #define isatty _isatty

    #define strncasecmp _strnicmp

    #define usleep(x) Sleep((x)/1000)

    // Call process.h:int _getpid() without having to include the header.
    int windows_getpid();

    // Call io.h:_isatty() without having to include the header.
    int windows_isatty(int fd_);
#endif

#ifdef _MSC_VER
    #define SG_FMT_SIZE_T "Iu"
#else
    #define SG_FMT_SIZE_T "zu"
#endif

#endif

/* eof MSCFixes.h */
