/*!
 * \file    MTDomainID.h
 * \brief   Define an MTDomainID.
 * \date    Copyright 2017 ARM Limited. All rights reserved.
 *
 * This defines a private interface for the use of by ARM.  The interface is
 * subject to change.
 *
 */
#ifndef ARM_INCLUDE_sg_MTDomainID_h
#define ARM_INCLUDE_sg_MTDomainID_h

#include "stdint.h"
#include <iosfwd>

namespace sg
{
    /// An ID that represents an MTDomain.
    ///
    /// ID 0 is special and means an unknown MTDomain.
    class MTDomainID {
    public:

        /// Construct the unknown MTDomainID.
        MTDomainID() : id(0) {}
    
        explicit MTDomainID(unsigned id_) : id(id_) {}
    
        bool operator ==(MTDomainID o_) const { return id == o_.id;    }
        bool operator !=(MTDomainID o_) const { return !(*this == o_); }

        /// Make sortable
        bool operator < (MTDomainID o_) const { return id < o_.id; }

        bool isUnknown() const { return id == 0; }

        /// The maximum MTDomainID supported.
        static unsigned const MAX = 255;

        void print(std::ostream& os_) const;

        unsigned as_unsigned() const { return id; }
    
    private:
        // We are not expecting hundreds of MTDomains, so just use a small number for the moment.
        uint8_t      id;
    };

    std::ostream& operator <<(std::ostream& os_, MTDomainID t_);
}

#endif
