/*!
 * \file    PPU.h
 * \brief   Defines power state modes
 * \date    Copyright 2016 ARM Limited. All rights reserved.
 *
 */

#ifndef ARM_PPU_H
#define ARM_PPU_H

#include <string>

namespace sg
{

    struct PModes
    {
        enum prequest_enum {prequest_OFF=0, prequest_ON=0x8};  //defines ON/OFF state sent on prequest
        enum pactive_enum {pactive_OFF=0x1, pactive_ON=0x100}; //defines ON/OFF state sent on pactive 
    };
    
    inline const std::string PModeToString(uint32_t mode)
    {
        switch (mode)
        {
            case sg::PModes::prequest_OFF:
                return "prequest_OFF";
            case sg::PModes::prequest_ON:
                return "prequest_ON";
            case sg::PModes::pactive_OFF:
                return "pactive_OFF";
            case sg::PModes::pactive_ON:
                return "pactive_ON";
            default:
                return std::to_string(mode);
        }
    }
}

#endif
