/*!
 * \file    SGFactory.h
 * \brief   Component factory types.
 * \date    Copyright 2005-2006, 2013 ARM Limited. All rights reserved.
 *
 */

#ifndef ARM_SG_FACTORY_H
#define ARM_SG_FACTORY_H


#include "SGBasicComponentFactory.h"

namespace sg {

class FSComponentFactory;
class Component;
class SimulationContext;

class FSComponentFactory : public BasicComponentFactory
{
 protected:
    int ref_count;

 public:
    FSComponentFactory() : ref_count(1) {
    }

    /*! Allow subclasses to be destructed through base class. */
    virtual ~FSComponentFactory() {
    }

    //! Do nothing when a client calls dereference: the factory is a
    //! singleton.
    virtual void dereference() {
        --ref_count;
        if (ref_count == 0) {
            delete this;
        }
    }

    //! Increase the reference count on a factory to stop it being deleted.
    void addReference() {
        ++ref_count;
    }

    /*! Return a string that uniquely identifies the class of components
     *  created by this factory.
     */
    virtual std::string getClassName() = 0;

    // Subclasses should implement base class method from sg::ComponentFactory:
    // virtual ComponentBase * instantiate(std::string const & instance_name,
    //                                     SimulationContext * context,
    //                                     Params const & params) = 0;
};

#define COMPONENT_FACTORY_FUNC(n) n##_GetFactory

/*! Declare the existence of a LISA factory function for the component. */
#define COMPONENT_FACTORY_DECL(n) sg::ComponentFactory * n##_GetFactory() ; sg::ComponentBase * n##_ComponentFactory(const char * instName, sg::SimulationContext *simulationContext, const sg::Params& params) ;

} // namespace sg

#endif // ARM_SG_FACTORY_H
