/*!
 * \file    SGPChannel.h
 * \brief   Protocol used to communicate power state changes
 * \date    Copyright 2016 ARM Limited. All rights reserved.
 *
 */

#ifndef ARM_SG_PCHANNEL_H
#define ARM_SG_PCHANNEL_H

#include "sg/SGPort.h"

namespace sg
{

    struct PChannel : public Port
    {
        enum presp_t    { ACCEPT, DENY };

        // To be implemented by a device
        declare_slave_behaviour(prequest, presp_t (uint32_t));

        // To be implemented by a power controller
        declare_master_behaviour(pactive, void (uint32_t));

        PChannel()
        {
            define_behaviour(prequest);
            prequest.bind_default_behaviour(this, &PChannel::default_prequest);

            define_behaviour(pactive);                
            pactive.bind_default_behaviour(this, &PChannel::default_pactive);
        }

        private:

        presp_t default_prequest(uint32_t)  { return DENY; }
        void default_pactive(uint32_t)  {}
    };
    
    inline const std::string PrespToString(uint32_t resp)
    {
        switch (resp)
        {
            case sg::PChannel::ACCEPT:
                return "ACCEPT";
            case sg::PChannel::DENY:
                return "DENY";
            default:
                return "unknown response";
        }
    }
}

#endif
