/*!
 *  \file   SizeAndConstructionProfiler.h
 *  \brief  Simple generic profiling class
 *  \date   Copyright ARM Limited 2013 All Rights Reserved.
 *
 * This file is not part of a stable public API and it may
 * change without notice.
 */

#ifndef INCLUDED_SG_SIZE_AND_CONSTRUCTION_PROFILER_H
#define INCLUDED_SG_SIZE_AND_CONSTRUCTION_PROFILER_H


#include <vector>
#include <cstddef>  // for size_t


namespace sg {


//! Generic class used to help profile class usage and in particular gather
//  statistics regarding the frequency of an integer "size" attribute
class SizeAndConstructionProfiler
{
private:
    const char*                     source_class;
    unsigned                        num_of_constructions;
    unsigned                        num_of_destructions;
    unsigned                        num_active;
    unsigned                        max_active;
    SizeAndConstructionProfiler*    next;
    std::vector<size_t>*            freq;   // Not aggregated to avoid destruction

    static SizeAndConstructionProfiler*  profiler_list;

    static void reportAll();

    void report();

public:
    SizeAndConstructionProfiler(const char* source_class_);

    void construct();
    void destruct();
    void sample(size_t elements);
};


extern SizeAndConstructionProfiler  fast_vector_profiler;
extern SizeAndConstructionProfiler  reference_counter_profiler;
extern SizeAndConstructionProfiler  fs_heap_profiler;


} // namespace sg

#endif
