/**
 *  \file   SysReg64RegisterNumber.h
 *  \brief  Encodings for the sysreg64 plugin register interface
 *  \date   Copyright ARM Limited 2012 All Rights Reserved.
 */
#ifndef FRAMEWORK_INCLUDE_SG_SYSREG64REGISTERNUMBER_H_0BB3507D
#define FRAMEWORK_INCLUDE_SG_SYSREG64REGISTERNUMBER_H_0BB3507D 1

#include <sg/PluginRegisterPublishingInterface.h>

namespace sg
{
    /**
     * Helper structure to encode / decode register numbers for the sysreg64 interface.
     *
     * Most fields are as used by the AArch64 MSR/MRS instructions.
     */
    struct SysReg64RegisterNumber
    {
        unsigned op0, op1, crn, crm, op2;

        /**
         * Register encodings as used by MSR/MRS instructions.
         */
        SysReg64RegisterNumber(
            unsigned op0_,
            unsigned op1_,
            unsigned crn_,
            unsigned crm_,
            unsigned op2_
        )
            : op0(op0_)
            , op1(op1_)
            , crn(crn_)
            , crm(crm_)
            , op2(op2_)
        {
        }

        /**
         * Decode a register number in to split out fields.
         */
        static SysReg64RegisterNumber decode(sg::PluginRegisterPublishingInterface::RegisterNumber register_number)
        {
            return SysReg64RegisterNumber(
                    field(   OP0_BIT, 2, register_number),
                    field(   OP1_BIT, 3, register_number),
                    field(   CRN_BIT, 4, register_number),
                    field(   CRM_BIT, 4, register_number),
                    field(   OP2_BIT, 3, register_number)
            );
        }

        /**
         * Encode the split out fields into a register number.
         */
        sg::PluginRegisterPublishingInterface::RegisterNumber encode() const
        {
            return ((op0    & 0x3) <<    OP0_BIT)
                 | ((op1    & 0x7) <<    OP1_BIT)
                 | ((crn    & 0xf) <<    CRN_BIT)
                 | ((crm    & 0xf) <<    CRM_BIT)
                 | ((op2    & 0x7) <<    OP2_BIT)
                 ;
        }

    private:
        enum {
               OP0_BIT =  0,
               OP1_BIT =  4,
               CRN_BIT =  8,
               CRM_BIT = 16,
               OP2_BIT = 20
        };

        static unsigned field(unsigned lsb, unsigned width, unsigned value)
        {
            return (value >> lsb) & ((1 << width) -1);
        }
    }; // class SysReg64RegisterNumber

} // namespace sg

#endif // FRAMEWORK_INCLUDE_SG_SYSREG64REGISTERNUMBER_H_0BB3507D
