/* -*- c++ -*- */
/* vim: syntax=cpp
 */
/*!
 *  \file   Time.h
 *  \brief  Time reporting for Wall and per Process clocks
 *  \date   Copyright ARM Limited 2008 All Rights Reserved.
 *          Copyright ARM Limited 2011 All Rights Reserved.
 *
 *  This file is not part of a stable public API and it may
 *  change without notice.
 */

#ifndef INCLUDED_SG_TIME_H
#define INCLUDED_SG_TIME_H

#include <cstdint>

namespace sg {

/*! Get Wall clock time in microseconds
 *
 * \param   tim     buffer to accept time
 * \return  0 on success
 */
int GetWallTime(uint64_t* tim);

/*! Fastsim Process Time usage */
struct ProcessTime
{
    /*! User time in microseconds */
    uint64_t utime;
    /*! System time in microseconds */
    uint64_t stime;
};

/*! Get Process Time Usage
 *
 *  \param  tim     buffer to accept time
 *  \return 0 on success
 */
int GetProcessTime(sg::ProcessTime* tim);

}

#endif

/* eof Time.h */
