/*!
 *  \file   UnorderedMap.h
 *  \brief  Includes unordered_map class header in a controlled manner.
 *  \date   Copyright ARM Limited 2014 All Rights Reserved.
 *
 *  The template class std::unordered_map<K, T> is a C++ (TR1 extension)
 *  collection that controls a varying-length sequence of elements of type
 *  std::pair<const K, T>.
 *  The sequence is weakly ordered by a hash function.
 *
 *  This class is available with both Visual Studio and GCC, but via possibly
 *  different headers.
 *
 *  This file is not part of a stable public API and it may
 *  change without notice.
 */


#ifndef INCLUDED_UNORDERED_MAP_H
#define INCLUDED_UNORDERED_MAP_H

#include "sg/PreInc.h"

#if defined(_MSC_VER)
    #include <unordered_map>
    #if ! defined(_HAS_CPP0X)

        /* Ensure unordered_map<> and hash<> come into std namespace. */
        namespace std {
            using tr1::unordered_map;
            using tr1::hash;
        }
    #endif
#elif defined(__clang__)
    #if __cplusplus >= 201103L
        #include <unordered_map>
    #else
        /* When compiling in C++03 mode, pull unordered_map<> and hash<> into std namespace */
        #include <tr1/unordered_map>
        namespace std {
            using tr1::unordered_map;
            using tr1::hash;
        }
    #endif
#elif __GNUC__ >= 4
    #if defined(__GXX_EXPERIMENTAL_CXX0X__) || __cplusplus >= 201103L
        #include <unordered_map>
    #else
        #include <tr1/unordered_map>

        /* Ensure unordered_map<> and hash<> come into std namespace. */
        namespace std {
            using tr1::unordered_map;
            using tr1::hash;
        }
    #endif
#else
    #error "Unsupported Compiler"
#endif

#include "sg/PostInc.h"

#endif

/* eof UnorderedMap.h */
