//
// (C) COPYRIGHT 2017 ARM Limited. ALL RIGHTS RESERVED.
//

#ifndef ARM_MODELS_TLMMISC_H
#define ARM_MODELS_TLMMISC_H

/*
 * AMBA TLM library inclusion but without SystemC-dependent headers.
 * Also defines some useful features which should possibly be ported back
 * to the ARM TLM library for future versions.
 *
 * This file is called TLMMisc.h to avoid conflict (on case insenstive
 * filesystems) with tlm.h.
 */

#include <iostream>

#include <ARM/TLM/arm_tlm_protocol.h>

namespace ARM
{
namespace TLM
{

/*
 * Ostream inserters for TLM protocol printing either the raw enumeration
 * strings (e.g. PROTOCOL_ACE_LITE) or the presentation strings (e.g. ACELite) if
 * the enum is wrapped in PresentationPrint
 */
template <typename EnumType>
class PresentationPrint
{
protected:
    EnumType value;

public:
    PresentationPrint(EnumType value_) : value(value_) {}

    operator EnumType () { return value; }
};

template <typename EnumType>
PresentationPrint<EnumType> presentation(EnumType value)
{ return PresentationPrint<EnumType>(value); }

std::ostream& operator <<(std::ostream&, ARM::TLM::Protocol protocol);
std::ostream& operator <<(std::ostream&,
    PresentationPrint<ARM::TLM::Protocol> protocol);

}
}

#endif // ARM_MODELS_TLMMISC_H
