//-------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited.
//
//            (C) COPYRIGHT 2016 ARM Limited.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited.
//
// Please refer to your legal agreement with ARM Limited to
// understand what you are licensed to do with this example code.
//-------------------------------------------------------------------

#ifndef ARM_AXI_PIN_TO_TLM_HXX
#define ARM_AXI_PIN_TO_TLM_HXX

#include "ARM/TLM/arm_tlm_socket.h"
#include "ARM/TLM/arm_axi4_socket.h"
#include "tlm_utils/simple_initiator_socket.h"
#include "tlm_utils/simple_target_socket.h"
#include "SCAXI4.h"

#include <stdint.h>

#include "cm_sysc_utils.h"

namespace ARM
{
namespace Models
{
class Name;
namespace SignalLevel
{
class AXI4SignalAccessMasterAdapter;
class AXI4Master;
}
}
}

namespace ARM
{
namespace Models
{
namespace Cycle
{
namespace Adapters
{

template <typename ADDR=sc_uint<44>,typename DATA=sc_biguint<128>,typename ID=sc_uint<9>> 
class axi_pin_to_tlm : public sc_core::sc_module
{
  SC_HAS_PROCESS(axi_pin_to_tlm);

private:
  const bool debug_ports;
public:
  sc_core::sc_in_clk clk;
  
public:
  axi_pin_to_tlm(const sc_core::sc_module_name &name,ARM::TLM::Protocol protocol=ARM::TLM::PROTOCOL_ACE): 
      sc_core::sc_module(name),debug_ports(false),clk("CLK_IN")
  // AW Channel
  ,AWIDport("AWIDport")
  ,AWADDRport("AWADDRport")
  ,AWLENport("AWLENport")
  ,AWSIZEport("AWSIZEport")
  ,AWBURSTport("AWBURSTport")
  ,AWLOCKport("AWLOCKport")
  ,AWCACHEport("AWCACHEport")
  ,AWPROTport("AWPROTport")
  ,AWVALIDport("AWVALIDport")
  ,AWREADYport("AWREADYport")
  
  // W channel
  ,WDATAport("WDATAport")
  ,WSTRBport("WSTRBport")
  ,WLASTport("WLASTport")
  ,WVALIDport("WVALIDport")
  ,WREADYport("WREADYport")
  
  // B Channel  
  ,BIDport("BIDport")
  ,BRESPport("BRESPport")
  ,BVALIDport("BVALIDport")
  ,BREADYport("BREADYport")
  
  // AR channel
  ,ARIDport("ARIDport")
  ,ARADDRport("ARADDRport")
  ,ARLENport("ARLENport")
  ,ARSIZEport("ARSIZEport")
  ,ARBURSTport("ARBURSTport")
  ,ARLOCKport("ARLOCKport")
  ,ARCACHEport("ARCACHEport")
  ,ARPROTport("ARPROTport")
  ,ARVALIDport("ARVALIDport")
  ,ARREADYport("ARREADYport")
  
  // R channel
  ,RIDport("RIDport")
  ,RDATAport("RDATAport")
  ,RRESPport("RRESPport")
  ,RLASTport("RLASTport")
  ,RVALIDport("RVALIDport")
  ,RREADYport("RREADYport")
      
  // ACE AR additions
  ,ARSNOOPport("ARSNOOPport")
  ,ARDOMAINport("ARDOMAINport")
  ,ARBARport("ARBARport")
  
  // ACE AW additions
  ,AWSNOOPport("AWSNOOPport")
  ,AWDOMAINport("AWDOMAINport")
  ,AWBARport("AWBARport")
  ,AWUNIQUEport("AWUNIQUEport")
      
  // ACE AC channel   
  ,ACVALIDport("ACVALIDport")
  ,ACREADYport("ACREADYport")
  ,ACADDRport("ACADDRport")
  ,ACSNOOPport("ACSNOOPport")
  ,ACPROTport("ACPROTport")
  
  // ACE CR channel
  ,CRVALIDport("CRVALIDport")
  ,CRREADYport("CRREADYport")
  ,CRRESPport("CRRESPport")
  
  // ACE CD Channel
  ,CDVALIDport("CDVALIDport")
  ,CDREADYport("CDREADYport")
  ,CDDATAport("CDDATAport")
  ,CDLASTport("CDLASTport")
 
  ,RACKport("RACKport")
  ,WACKport("WACKport")
  
  // These signals are optional and not currently used
  //,AWQOSport("AWQOSport")
  //,AWREGIONport("AWREGIONport")
  //,AWUSERport("AWUSERport")
  //,RUSERport("RUSERport")
  //,WIDport("WIDport")
  //,WUSERport("WUSERport")
  //,BUSERport("BUSERport")
  //,ARQOSport("ARQOSport")
  //,ARREGIONport("ARREGIONport")
  //,ARUSERport("ARUSERport")    
  {
    SC_METHOD(posedge);
    sensitive << clk.pos();
    dont_initialize();
  
    SC_METHOD(negedge);
    sensitive << clk.neg();
    dont_initialize();
  
    std::ostream *debug_stream = (debug_ports ? &std::cout : NULL);
    
    kACADDR = new ScOutToPort<sc_out<ADDR> > (&ACADDRport);
    kACPROT = new ScOutToPort<sc_out<sc_uint<3> > > (&ACPROTport);
    kACSNOOP = new ScOutToPort<sc_out<sc_uint<4> > > (&ACSNOOPport);
    kACVALID = new ScOutToPort<sc_out<bool > > (&ACVALIDport);
    kARREADY = new ScOutToPort<sc_out<bool > > (&ARREADYport);
    kAWREADY = new ScOutToPort<sc_out<bool > > (&AWREADYport);
    kBID = new ScOutToPort<sc_out<ID> > (&BIDport);
    kBRESP = new ScOutToPort<sc_out<sc_uint<2> > > (&BRESPport);
    kBVALID = new ScOutToPort<sc_out<bool > > (&BVALIDport);
    kCDREADY = new ScOutToPort<sc_out<bool > > (&CDREADYport);
    kCRREADY = new ScOutToPort<sc_out<bool > > (&CRREADYport);
    kRDATA = new ScOutToPort<sc_out<DATA> > (&RDATAport);
    kRID = new ScOutToPort<sc_out<ID > > (&RIDport);
    kRLAST = new ScOutToPort<sc_out<bool > > (&RLASTport);
    kRRESP = new ScOutToPort<sc_out<sc_uint<4> > > (&RRESPport);
    kRVALID = new ScOutToPort<sc_out<bool > > (&RVALIDport);
    kWREADY = new ScOutToPort<sc_out<bool > > (&WREADYport);
    kACREADY = new PortToScIn<sc_in<bool > > (&ACREADYport);
    kARADDR = new PortToScIn<sc_in<ADDR> > (&ARADDRport);
    kARBURST = new PortToScIn<sc_in<sc_uint<2> > > (&ARBURSTport);
    kARCACHE = new PortToScIn<sc_in<sc_uint<4> > > (&ARCACHEport);
    kARDOMAIN = new PortToScIn<sc_in<sc_uint<2> > > (&ARDOMAINport);
    kARID = new PortToScIn<sc_in<ID > > (&ARIDport);
    kARLEN = new PortToScIn<sc_in<sc_uint<8> > > (&ARLENport);
    kARLOCK = new PortToScIn<sc_in<bool > > (&ARLOCKport);
    kARPROT = new PortToScIn<sc_in<sc_uint<3> > > (&ARPROTport);
    kARSIZE = new PortToScIn<sc_in<sc_uint<3> > > (&ARSIZEport);
    kARSNOOP = new PortToScIn<sc_in<sc_uint<4> > > (&ARSNOOPport);
    kARVALID = new PortToScIn<sc_in<bool > > (&ARVALIDport);
    kAWADDR = new PortToScIn<sc_in<ADDR> > (&AWADDRport);
    kAWBURST = new PortToScIn<sc_in<sc_uint<2> > > (&AWBURSTport);
    kAWCACHE = new PortToScIn<sc_in<sc_uint<4> > > (&AWCACHEport);
    kAWDOMAIN = new PortToScIn<sc_in<sc_uint<2> > > (&AWDOMAINport);
    kAWID = new PortToScIn<sc_in<ID> > (&AWIDport);
    kAWLEN = new PortToScIn<sc_in<sc_uint<8> > > (&AWLENport);
    kAWLOCK = new PortToScIn<sc_in<bool > > (&AWLOCKport);
    kAWPROT = new PortToScIn<sc_in<sc_uint<3> > > (&AWPROTport);
    kAWSIZE = new PortToScIn<sc_in<sc_uint<3> > > (&AWSIZEport);
    kAWSNOOP = new PortToScIn<sc_in<sc_uint<3> > > (&AWSNOOPport);
    kAWUNIQUE = new PortToScIn<sc_in<bool > > (&AWUNIQUEport);
    kAWVALID = new PortToScIn<sc_in<bool > > (&AWVALIDport);
    kBREADY = new PortToScIn<sc_in<bool > > (&BREADYport);
    kCDDATA = new PortToScIn<sc_in<DATA> > (&CDDATAport);
    kCDLAST = new PortToScIn<sc_in<bool > > (&CDLASTport);
    kCDVALID = new PortToScIn<sc_in<bool > > (&CDVALIDport);
    kCRRESP = new PortToScIn<sc_in<sc_uint<5> > > (&CRRESPport);
    kCRVALID = new PortToScIn<sc_in<bool > > (&CRVALIDport);
    kRACK = new PortToScIn<sc_in<bool > > (&RACKport);
    kRREADY = new PortToScIn<sc_in<bool > > (&RREADYport);
    kWACK = new PortToScIn<sc_in<bool > > (&WACKport);
    kWDATA = new PortToScIn<sc_in<DATA> > (&WDATAport);
    kWLAST = new PortToScIn<sc_in<bool > > (&WLASTport);
    kWSTRB = new PortToScIn<sc_in<sc_uint<16> > > (&WSTRBport);
    kWVALID = new PortToScIn<sc_in<bool > > (&WVALIDport);
    kAWBAR = new PortToScIn<sc_in<sc_uint<2> > > (&AWBARport);
    kARBAR = new PortToScIn<sc_in<sc_uint<2> > > (&ARBARport);
    
    ARM::Models::SignalLevel::SignalAccessMap signalSet;    
    signalSet.add_signal("ACADDR", kACADDR);
    signalSet.add_signal("ACPROT", kACPROT);
    signalSet.add_signal("ACSNOOP", kACSNOOP);
    signalSet.add_signal("ACVALID", kACVALID);
    signalSet.add_signal("ARREADY", kARREADY);
    signalSet.add_signal("AWREADY", kAWREADY);
    signalSet.add_signal("BID", kBID);
    signalSet.add_signal("BRESP", kBRESP);
    signalSet.add_signal("BVALID", kBVALID);
    signalSet.add_signal("CDREADY", kCDREADY);
    signalSet.add_signal("CRREADY", kCRREADY);
    signalSet.add_signal("RDATA", kRDATA);
    signalSet.add_signal("RID", kRID);
    signalSet.add_signal("RLAST", kRLAST);
    signalSet.add_signal("RRESP", kRRESP);
    signalSet.add_signal("RVALID", kRVALID);
    signalSet.add_signal("WREADY", kWREADY);
    signalSet.add_signal("ACREADY", kACREADY);
    signalSet.add_signal("ARADDR", kARADDR);
    signalSet.add_signal("ARBURST", kARBURST);
    signalSet.add_signal("ARCACHE", kARCACHE);
    signalSet.add_signal("ARDOMAIN", kARDOMAIN);
    signalSet.add_signal("ARID", kARID);
    signalSet.add_signal("ARLEN", kARLEN);
    signalSet.add_signal("ARLOCK", kARLOCK);
    signalSet.add_signal("ARPROT", kARPROT);
    signalSet.add_signal("ARSIZE", kARSIZE);
    signalSet.add_signal("ARSNOOP", kARSNOOP);
    signalSet.add_signal("ARVALID", kARVALID);
    signalSet.add_signal("AWADDR", kAWADDR);
    signalSet.add_signal("AWBURST", kAWBURST);
    signalSet.add_signal("AWCACHE", kAWCACHE);
    signalSet.add_signal("AWDOMAIN", kAWDOMAIN);
    signalSet.add_signal("AWID", kAWID);
    signalSet.add_signal("AWLEN", kAWLEN);
    signalSet.add_signal("AWLOCK", kAWLOCK);
    signalSet.add_signal("AWPROT", kAWPROT);
    signalSet.add_signal("AWSIZE", kAWSIZE);
    signalSet.add_signal("AWSNOOP", kAWSNOOP);
    signalSet.add_signal("AWUNIQUE", kAWUNIQUE);
    signalSet.add_signal("AWVALID", kAWVALID);
    signalSet.add_signal("BREADY", kBREADY);
    signalSet.add_signal("CDDATA", kCDDATA);
    signalSet.add_signal("CDLAST", kCDLAST);
    signalSet.add_signal("CDVALID", kCDVALID);
    signalSet.add_signal("CRRESP", kCRRESP);
    signalSet.add_signal("CRVALID", kCRVALID);
    signalSet.add_signal("RACK", kRACK);
    signalSet.add_signal("RREADY", kRREADY);
    signalSet.add_signal("WACK", kWACK);
    signalSet.add_signal("WDATA", kWDATA);
    signalSet.add_signal("WLAST", kWLAST);
    signalSet.add_signal("WSTRB", kWSTRB);
    signalSet.add_signal("WVALID", kWVALID);    
    signalSet.add_signal("ARBAR", kARBAR);    
    signalSet.add_signal("AWBAR", kAWBAR);    
    
    master_if = new SignalLevel::AXI4SignalAccessMasterAdapter();
    SignalLevel::AXI4PortParams& port_params = master_if->params;
    port_params.set_protocol(protocol);
    ADDR aw;
    port_params.set_addr_width(aw.length());
    DATA dw;
    port_params.set_data_width(dw.length());
    port_params.set_qos_width(4);
    ID iw;
    port_params.set_rid_width(iw.length());
    port_params.set_wid_width(iw.length());
    master_port = new SCAXI4::AXI4Master("MASTER_PORT",port_params.get_protocol());
    SignalLevel::AXI4InfixNameMapping name_mapping("","",true);
    master_if->bind_to_signals(signalSet, name_mapping,debug_stream);
    master_if->tie_off_signals(debug_stream);
    if (debug_ports) 
      master_if->print_binding_report(*debug_stream);
    master_port->set_signal_interface(*master_if);    
    
    // TLM socket constructors
    iSkt_master = new TLM::BaseMasterSocket<AXI4::ProtocolType>("iSkt_master",port_params.get_protocol(),port_params.get_data_width());        

    // TLM socket bindings
    master_port->bind_to_systemc_parent(*iSkt_master);
  }
  ~axi_pin_to_tlm() 
  {
    delete master_if;
    delete master_port;
    delete iSkt_master;
    
    delete kACADDR;
    delete kACPROT;
    delete kACSNOOP;
    delete kACVALID;
    delete kARREADY;
    delete kAWREADY;
    delete kBID;
    delete kBRESP;
    delete kBVALID;
    delete kCDREADY;
    delete kCRREADY;
    delete kRDATA;
    delete kRID;
    delete kRLAST;
    delete kRRESP;
    delete kRVALID;
    delete kWREADY;
    delete kACREADY;
    delete kARADDR;
    delete kARBURST;
    delete kARCACHE;
    delete kARDOMAIN;
    delete kARID;
    delete kARLEN;
    delete kARLOCK;
    delete kARPROT;
    delete kARSIZE;
    delete kARSNOOP;
    delete kARVALID;
    delete kAWADDR;
    delete kAWBURST;
    delete kAWCACHE;
    delete kAWDOMAIN;
    delete kAWID;
    delete kAWLEN;
    delete kAWLOCK;
    delete kAWPROT;
    delete kAWSIZE;
    delete kAWSNOOP;
    delete kAWUNIQUE;
    delete kAWVALID;
    delete kBREADY;
    delete kCDDATA;
    delete kCDLAST;
    delete kCDVALID;
    delete kCRRESP;
    delete kCRVALID;
    delete kRACK;
    delete kRREADY;
    delete kWACK;
    delete kWDATA;
    delete kWLAST;
    delete kWSTRB;
    delete kWVALID;
    delete kAWBAR;
    delete kARBAR;
  };
  
  // Master socket declarations
  TLM::BaseMasterSocket<AXI4::ProtocolType> *iSkt_master;
  
private:    
  // master port declarations
  SignalLevel::AXI4SignalAccessMasterAdapter *master_if;
  SCAXI4::AXI4Master* master_port;
  
  //signals to ports
  ScOutToPort<sc_out<ADDR> > *kACADDR;
  ScOutToPort<sc_out<sc_uint<3> > > *kACPROT;
  ScOutToPort<sc_out<sc_uint<4> > > *kACSNOOP;
  ScOutToPort<sc_out<bool > > *kACVALID;
  ScOutToPort<sc_out<bool > > *kARREADY;
  ScOutToPort<sc_out<bool > > *kAWREADY;
  ScOutToPort<sc_out<ID> > *kBID;
  ScOutToPort<sc_out<sc_uint<2> > > *kBRESP;
  ScOutToPort<sc_out<bool > > *kBVALID;
  ScOutToPort<sc_out<bool > > *kCDREADY;
  ScOutToPort<sc_out<bool > > *kCRREADY;
  ScOutToPort<sc_out<DATA> > *kRDATA;
  ScOutToPort<sc_out<ID > > *kRID;
  ScOutToPort<sc_out<bool > > *kRLAST;
  ScOutToPort<sc_out<sc_uint<4> > > *kRRESP;
  ScOutToPort<sc_out<bool > > *kRVALID;
  ScOutToPort<sc_out<bool > > *kWREADY;
  PortToScIn<sc_in<bool > > *kACREADY;
  PortToScIn<sc_in<ADDR> > *kARADDR;
  PortToScIn<sc_in<sc_uint<2> > > *kARBURST;
  PortToScIn<sc_in<sc_uint<4> > > *kARCACHE;
  PortToScIn<sc_in<sc_uint<2> > > *kARDOMAIN;
  PortToScIn<sc_in<ID > > *kARID;
  PortToScIn<sc_in<sc_uint<8> > > *kARLEN;
  PortToScIn<sc_in<bool > > *kARLOCK;
  PortToScIn<sc_in<sc_uint<3> > > *kARPROT;
  PortToScIn<sc_in<sc_uint<3> > > *kARSIZE;
  PortToScIn<sc_in<sc_uint<4> > > *kARSNOOP;
  PortToScIn<sc_in<bool > > *kARVALID;
  PortToScIn<sc_in<ADDR> > *kAWADDR;
  PortToScIn<sc_in<sc_uint<2> > > *kAWBURST;
  PortToScIn<sc_in<sc_uint<4> > > *kAWCACHE;
  PortToScIn<sc_in<sc_uint<2> > > *kAWDOMAIN;
  PortToScIn<sc_in<ID> > *kAWID;
  PortToScIn<sc_in<sc_uint<8> > > *kAWLEN;
  PortToScIn<sc_in<bool > > *kAWLOCK;
  PortToScIn<sc_in<sc_uint<3> > > *kAWPROT;
  PortToScIn<sc_in<sc_uint<3> > > *kAWSIZE;
  PortToScIn<sc_in<sc_uint<3> > > *kAWSNOOP;
  PortToScIn<sc_in<bool > > *kAWUNIQUE;
  PortToScIn<sc_in<bool > > *kAWVALID;
  PortToScIn<sc_in<bool > > *kBREADY;
  PortToScIn<sc_in<DATA> > *kCDDATA;
  PortToScIn<sc_in<bool > > *kCDLAST;
  PortToScIn<sc_in<bool > > *kCDVALID;
  PortToScIn<sc_in<sc_uint<5> > > *kCRRESP;
  PortToScIn<sc_in<bool > > *kCRVALID;
  PortToScIn<sc_in<bool > > *kRACK;
  PortToScIn<sc_in<bool > > *kRREADY;
  PortToScIn<sc_in<bool > > *kWACK;
  PortToScIn<sc_in<DATA> > *kWDATA;
  PortToScIn<sc_in<bool > > *kWLAST;
  PortToScIn<sc_in<sc_uint<16> > > *kWSTRB;
  PortToScIn<sc_in<bool > > *kWVALID;
  PortToScIn<sc_in<sc_uint<2> > > *kARBAR;
  PortToScIn<sc_in<sc_uint<2> > > *kAWBAR;
  
public:
  // ports 
  sc_in<ID> AWIDport;
  sc_in<ADDR> AWADDRport;
  sc_in<sc_uint<8> > AWLENport;
  sc_in<sc_uint<3> > AWSIZEport;
  sc_in<sc_uint<2> > AWBURSTport;
  sc_in<bool > AWLOCKport;
  sc_in<sc_uint<4> > AWCACHEport;
  sc_in<sc_uint<3> > AWPROTport;
  sc_in<bool > AWVALIDport;
  sc_out<bool > AWREADYport;
  
  sc_in<DATA> WDATAport;
  sc_in<sc_uint<16> > WSTRBport;
  sc_in<bool > WLASTport;
  sc_in<bool > WVALIDport;
  sc_out<bool > WREADYport;
  
  sc_out<ID> BIDport;
  sc_out<sc_uint<2> > BRESPport;
  sc_out<bool > BVALIDport;
  sc_in<bool > BREADYport;

  sc_in<ID > ARIDport;
  sc_in<ADDR> ARADDRport;
  sc_in<sc_uint<8> > ARLENport;
  sc_in<sc_uint<3> > ARSIZEport;
  sc_in<sc_uint<2> > ARBURSTport;
  sc_in<bool > ARLOCKport;
  sc_in<sc_uint<4> > ARCACHEport;
  sc_in<sc_uint<3> > ARPROTport;
  sc_in<bool > ARVALIDport;
  sc_out<bool > ARREADYport;

  sc_out<ID > RIDport;
  sc_out<DATA> RDATAport;
  sc_out<sc_uint<4> > RRESPport;
  sc_out<bool > RLASTport;
  sc_out<bool > RVALIDport;
  sc_in<bool > RREADYport;
    
  sc_in<sc_uint<4> > ARSNOOPport;
  sc_in<sc_uint<2> > ARDOMAINport;  
  sc_in<sc_uint<2> > ARBARport;  

  sc_in<sc_uint<3> > AWSNOOPport;
  sc_in<sc_uint<2> > AWDOMAINport;
  sc_in<sc_uint<2> > AWBARport;
  sc_in<bool > AWUNIQUEport;  

  sc_out<bool > ACVALIDport;
  sc_in<bool > ACREADYport;
  sc_out<ADDR> ACADDRport;
  sc_out<sc_uint<4> > ACSNOOPport;
  sc_out<sc_uint<3> > ACPROTport;

  sc_in<bool > CRVALIDport;
  sc_out<bool > CRREADYport;
  sc_in<sc_uint<5> > CRRESPport;
  
  sc_in<bool > CDVALIDport;
  sc_out<bool > CDREADYport;
  sc_in<DATA> CDDATAport;
  sc_in<bool > CDLASTport;
  
  sc_in<bool > RACKport;
  sc_in<bool > WACKport;

/*  
  sc_in<bool > AWAKEUPport;
  sc_out<bool > ACLKENport;
  sc_out<sc_uint<4> > ACVMIDEXTport;
  sc_out<bool > ACWAKEUPport;
  sc_out<bool > SCLKport;
  sc_out<bool > SYSCOACKport;
  sc_out<bool > nSRESETport;
  sc_in<sc_uint<4> > ARVMIDEXTport;
  sc_in<bool > SYSCOREQport;
*/  
  void posedge()
  {
    master_port->posedge();
  }
  void negedge()
  {
    master_port->negedge();
  }

};

}
}
}
}

#endif // ARM_MODELS_CYCLE_Kite_Kite_IMP_H
