//-------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited.
//
//            (C) COPYRIGHT 2016 ARM Limited.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited.
//
// Please refer to your legal agreement with ARM Limited to
// understand what you are licensed to do with this example code.
//-------------------------------------------------------------------
#ifndef _CM_SYSC_UTIL_H_
#define _CM_SYSC_UTIL_H_
template <typename T> int sc_len(T t) { return t.length(); }

template <> inline int sc_len(bool d) { return 1; }

template <typename T> int getLength(T *p)
{
  typename T::data_type d = 0;
  return sc_len<typename T::data_type>(d);
}

template <typename T> int my_to_int(T x, int last, int first) { return x.range(last, first).to_int(); }

template <> inline int my_to_int(bool x, int, int) { return int(x); }

template <typename T> void my_write(T &x, int last, int first, uint8_t val) { x.range(last, first) = val; }

template <> inline void my_write(bool &x, int last, int first, uint8_t val) { x = val; }

template <typename portType> class ScOutToSignal : public ARM::Models::SignalLevel::SignalAccess
{
public:
  typedef sc_signal<typename portType::data_type> signalType;
  ScOutToSignal(portType *p, signalType *s) : ARM::Models::SignalLevel::SignalAccess(getLength(p))
  {
    port = p;
    signal = s;
    port->bind(*s);
  }

  void get_ptr(uint8_t *value) const
  {
    typename portType::data_type x = signal->read();
    unsigned int bytes = (sc_len(x) + 7u) / 8;
    int j = 0;
    for (unsigned int i = 0; i < bytes; i++, j += 8)
    {
      int first = j;
      int last = std::min(sc_len(x) - 1, j + 7);
      value[i] = static_cast<uint8_t>(my_to_int(x, last, first));
    }
  }

  void set_ptr(uint8_t *xx) {}

  ~ScOutToSignal(){};

private:
  portType *port;
  signalType *signal;
};

template <typename portType> class SignalToScIn : public ARM::Models::SignalLevel::SignalAccess
{
public:
  typedef sc_signal<typename portType::data_type> signalType;

  SignalToScIn(portType *p, signalType *s) : ARM::Models::SignalLevel::SignalAccess(getLength(p))
  {
    port = p;
    signal = s;
    port->bind(*s);
  }

  void set_ptr(uint8_t *value)
  {

    typename portType::data_type out = 0;

    unsigned int bytes = (sc_len(out) + 7u) / 8;

    int j = 0;
    for (unsigned int i = 0; i < bytes; i++, j += 8)
    {
      int first = j;
      int last = std::min(sc_len(out) - 1, j + 7);
      my_write(out, last, first, value[i]);
    }
    signal->write(out);
  }

  ~SignalToScIn() {}

  void get_ptr(uint8_t *value) const {}

private:
  portType *port;
  signalType *signal;
};

// this class is used by axi_pin_to_tlm and axi_tlm_to_pin
template <typename portType> class ScOutToPort : public ARM::Models::SignalLevel::SignalAccess
{
public:
  ScOutToPort(portType *p) : ARM::Models::SignalLevel::SignalAccess(getLength(p))
  {
    port = p;
  }
  ~ScOutToPort(){};

  void get_ptr(uint8_t *value) const
  {
    //printf("*ScOutToPort - get_ptr (%s) (%d)\n",port->name(),value[0]);
  }

  void set_ptr(uint8_t *value) 
  {
    // if not bound don't write to it
    if (port->size() == 0)
      return;

    typename portType::data_type out = 0;

    unsigned int bytes = (sc_len(out) + 7u) / 8;

    int j = 0;
    for (unsigned int i = 0; i < bytes; i++, j += 8)
    {
      int first = j;
      int last = std::min(sc_len(out) - 1, j + 7);
      my_write(out, last, first, value[i]);
    }
    port->write(out);
    //printf("ScOutToPort - set_ptr (%s) (%d)\n",port->name(),value[0]);
  }

private:
  portType *port;
};

// this class is used by axi_pin_to_tlm and axi_tlm_to_pin
template <typename portType> class PortToScIn : public ARM::Models::SignalLevel::SignalAccess
{
public:
  PortToScIn(portType *p) : ARM::Models::SignalLevel::SignalAccess(getLength(p))
  {
    port = p;
  }
  ~PortToScIn() {}

  void set_ptr(uint8_t *value)
  {
//    printf("*PortToScIn - set_ptr (%s) (%d)\n",port->name(),value[0]);   
  }

  void get_ptr(uint8_t *value) const 
  {
    typename portType::data_type x = 0;
    if (port->size() > 0)
    {
       x = port->read();
    }

    unsigned int bytes = (sc_len(x) + 7u) / 8;
    int j = 0;
    for (unsigned int i = 0; i < bytes; i++, j += 8)
    {
      int first = j;
      int last = std::min(sc_len(x) - 1, j + 7);
      value[i] = static_cast<uint8_t>(my_to_int(x, last, first));
    }
    //printf("PortToScIn - get_ptr (%s) (%d)\n",port->name(),value[0]);
  }

private:
  portType *port;
};
#endif
