//-------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited.
//
//            (C) COPYRIGHT 2015 ARM Limited.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited.
//
// This file forms part of the ARM standard for performance model
// memory transactions for SystemC TLM 2.0 and should not be
// modified.
//
// This standard is currently under review and is subject to change.
//-------------------------------------------------------------------

#ifndef ARM_AXI4_PHASE_H
#define ARM_AXI4_PHASE_H

#include <stdint.h>

namespace ARM
{
namespace AXI4
{

/**
 * Communication phases for AXI4.
 *
 * Phases correspond to events on AXI4 channels.
 *
 * VALID phases are used when offering a Payload to another agent. READY is
 * used as a response (either immediately/in the same cycle with: phase =
 * ...READY; return TLM_UPDATED, or later with another communicating function
 * call) to a VALID phase. VALID_LAST phases are VALID phases used for last
 * data beats.
 */
enum Phase
{
    /*
     * Bit position meaning:
     * [0] == 1: READY
     * [7:4]: Channel number
     * [7:4] == 0: UNINITIALIZED
     * [7:4] == 1: AW
     * [7:4] == 2: W
     * [7:4] == 3: B
     * [7:4] == 4: AR
     * [7:4] == 5: R
     * [7:4] == 6: AC
     * [7:4] == 7: CR
     * [7:4] == 8: CD
     * [7:4] == 9: WACK
     * [7:4] == A: RACK
     * [8] == 1: LAST
     */

    PHASE_UNINITIALIZED = 0x000,

    AW_VALID      = 0x010,
    AW_READY      = 0x011,

    W_VALID       = 0x020,
    W_VALID_LAST  = 0x120,
    W_READY       = 0x021,

    B_VALID       = 0x030,
    B_READY       = 0x031,

    AR_VALID      = 0x040,
    AR_READY      = 0x041,

    R_VALID       = 0x050,
    R_VALID_LAST  = 0x150,
    R_READY       = 0x051,

    AC_VALID      = 0x060,
    AC_READY      = 0x061,

    CR_VALID      = 0x070,
    CR_READY      = 0x071,

    CD_VALID      = 0x080,
    CD_VALID_LAST = 0x180,
    CD_READY      = 0x081,

    WACK          = 0x090,
    RACK          = 0x0A0

};

}
}

#endif // ARM_AXI4_PHASE_H
