//-------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited.
//
//            (C) COPYRIGHT 2015 ARM Limited.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited.
//
// This file forms part of the ARM standard for performance model
// memory transactions for SystemC TLM 2.0 and should not be
// modified.
//
// This standard is currently under review and is subject to change.
//-------------------------------------------------------------------

#ifndef ARM_AXI4_SOCKET_H
#define ARM_AXI4_SOCKET_H

#include "arm_axi4_payload.h"
#include "arm_axi4_phase.h"
#include "arm_tlm_socket.h"

namespace ARM
{
namespace AXI4
{

/**
 * Payload and Phase grouped for use with socket templates in a similar
 * way to tlm::tlm_base_protocol_types.
 */
class ProtocolType
{
public:
    typedef Payload tlm_payload_type;
    typedef Phase tlm_phase_type;
};

/** ARM::TLM::SimpleMasterSocket specialised for AXI4 payloads/phases. */
template <typename Module, typename Types = ProtocolType>
class SimpleMasterSocket : public ARM::TLM::SimpleMasterSocket <Module, Types>
{
private:
    typedef typename ARM::TLM::SimpleMasterSocket<Module, Types> BaseType;

public:
    SimpleMasterSocket(const char* name_, Module& t,
        typename BaseType::NBFunc bw,
        TLM::Protocol protocol_, unsigned width_) :
        TLM::SimpleMasterSocket <Module, Types>(name_, t, bw, protocol_, width_)
    {}
};

/** ARM::TLM::SimpleSlaveSocket specialised for AXI4 payloads/phases. */
template <typename Module, typename Types = ProtocolType>
class SimpleSlaveSocket : public ARM::TLM::SimpleSlaveSocket <Module, Types>
{
private:
    typedef typename ARM::TLM::SimpleSlaveSocket<Module, Types> BaseType;

public:
    SimpleSlaveSocket(const char* name_, Module& t,
        typename BaseType::NBFunc fw,
        TLM::Protocol protocol_, unsigned width_,
        typename BaseType::DebugFunc dbg = NULL) :
        TLM::SimpleSlaveSocket <Module, Types>(name_, t, fw,
            protocol_, width_, dbg)
    {}
};

}
}

#endif // ARM_AXI4_SOCKET_H
