//
// (C) COPYRIGHT 2017 ARM Limited. ALL RIGHTS RESERVED.
//

#ifndef SCX_DUMP_WAVEFORMS_H
#define SCX_DUMP_WAVEFORMS_H

class scx_dump_waveforms: public scx_wrapper_feature
{
 private:
  CarbonWaveID* mWaveID;
 public:  
  scx_dump_waveforms(int num_cpus, CarbonObjectID * carbonModelHandle,scx::scx_debug_target_if& target):
    scx_wrapper_feature(num_cpus,carbonModelHandle,target)
  {
    setName("WAVEFORMS");
    setDescription("Enable or disable waveform dumping");
    addEnableParameter();
    // create a parameter to set the timeunit, this takes SystemC time, default is picoseconds
    addParameter("WAVEFORM_TIMEUNIT",(int) SC_PS,"SystemC time unit for waveform dumping",0,5);
    // create a parameter to set the waveform type, either VCD or FSDB, default is VCD
    addParameter("WAVEFORM_TYPE","VCD","Type of the waveform to dump, either VCD or FSDB");
    mWaveID = NULL;
  }
    
  CarbonWaveID* getWaveID()
  {
    return mWaveID;
  }
  
  // flush waveforms at end of simulation
  void end_of_simulation()
  {
    if (mWaveID)
    {
      carbonDumpFlush(mWaveID);
      carbonDumpOff(mWaveID);
    }
  }

  void update()
  {
    // REMEMBER: Update won't get called unless this feature is enabled
    // Also even though update is called multiple times the code in the if clause only gets executed once.
    if (mWaveID == NULL)
    {
      std::string name = "arm_cm_";
      name += moduleName();
      
      // get SystemC time unit and convert to carbon time scale
      int timeunit;
      getParameter("WAVEFORM_TIMEUNIT",timeunit);
      CarbonTimescale carbonTimeScale = (CarbonTimescale) ((timeunit-5)*3);

      // determine waveform type (either VCD or FSDB)
      std::string waveform_type;
      getParameter("WAVEFORM_TYPE",waveform_type);
      if (waveform_type == "VCD")
      {
	name += ".vcd";
	mWaveID = carbonWaveInitVCD(handle(),name.c_str(),carbonTimeScale);
      }
      else if (waveform_type == "FSDB")
      {
	name += ".fsdb";
	mWaveID = carbonWaveInitFSDB(handle(),name.c_str(),carbonTimeScale);
      }
      else SCX_REPORT_ERROR(name.c_str(),"Invalid waveform type %s specified",waveform_type.c_str());
      
      if (mWaveID)
      {
	SCX_REPORT_INFO(moduleName(),"Dumping %s waveforms into %s",waveform_type.c_str(),name.c_str());
	const char* topModuleName = carbonDBGetTopLevelModuleName(carbonGetDB(handle()));
	carbonDumpVars(mWaveID,0,topModuleName);
	carbonDumpOn(mWaveID);
      }
    }
  }

  void enable()
  {
    scx_wrapper_feature::enable();
    if (mWaveID)
      carbonDumpOn(mWaveID);
  }
  
  void disable()
  {
    scx_wrapper_feature::disable();
    if (mWaveID)
    {
      carbonDumpFlush(mWaveID);
      carbonDumpOff(mWaveID);
    }
  }
};
#endif
