//
// (C) COPYRIGHT 2017 ARM Limited. ALL RIGHTS RESERVED
//

#ifndef SCX_FEATURE_PARAMETER_H
#define SCX_FEATURE_PARAMETER_H

// this to_string function is here because I can't get the MTI systemc compiler (sccom)
// to recognized std::to_string when building the execution_tb or DSM
namespace scx {
template <typename T> std::string to_string(const T& n)
{
  std::ostringstream oss;
  oss << n;
  return oss.str();
}
}

// This class holds a generic string parameter
class scx_feature_parameter
{
private:
  std::string mName;
  std::string mValue;
  std::string mDescription;
protected:
  // string2bool function to avoid FM runtime dependency for DSM
  int string2bool(std::string str, int& err)
  {
    err = 0;
    if (str == "true" || str == "TRUE" || str == "True")
      return 1;
    if (str == "false" || str == "FALSE" || str == "False")
      return 0;
    err = 1;
    return 0;
  }
  
public:
  scx_feature_parameter(const std::string& name,const std::string& value,const std::string& description)
  {
    mName = name;
    mValue = value;
    mDescription = description;
  }
  virtual ~scx_feature_parameter() {}

  const std::string& name() { return mName; }
  const std::string& description() { return mDescription; }
  void setDescription(const std::string& desc) { mDescription = desc; }
  virtual bool getValue(std::string& value) { value = mValue; return true; }
  virtual bool setValue(const std::string& value) { mValue = value; return true; }
};

#endif
