//
// (C) COPYRIGHT 2017 ARM Limited. ALL RIGHTS RESERVED
//

#ifndef SCX_NET_PARAMETER_H
#define SCX_NET_PARAMETER_H

#include "scx/scx_feature_parameter.h"

// This class holds a pointer to a net in the Cycle Model
// that allows the creation of debug paramters to get and set the
// net value.
class scx_net_parameter: public scx_feature_parameter
{
private:
  int mNumWords;    // number of 32 bit words in net
  CarbonNetID* mNet;
  CarbonObjectID* mHandle;
 public:
 scx_net_parameter(std::string name,std::string net_path,CarbonObjectID* handle): scx_feature_parameter(name,"","")
  {
    mHandle = handle;
    mNet = carbonFindNet(handle,net_path.c_str());
    mNumWords = carbonGetNumUInt32s(mNet);
  }
  
  bool getValue(std::string& value)
  {
    if (mNumWords == 1)
    {
      CarbonUInt32 val32;
      carbonExamine(mHandle,mNet,&val32,NULL);
      value = scx::to_string(val32);
      return true;
    }
    if (mNumWords == 2)
    {
      CarbonUInt64 val64;
      carbonExamine(mHandle,mNet,(CarbonUInt32*) &val64,NULL);
      value = scx::to_string(val64);
      return true;
    }
    std::string msg = "Parameter value greater then 64 bits: " + name();
    SC_REPORT_WARNING(ARM_CM_SC_MSG_ID,msg.c_str());
    return false;    
  }

  bool setValue(const std::string& value)
  {
    if (mNumWords == 1)
    {
      // check for a boolean string
      int err;
      CarbonUInt32 val32 = string2bool(value,err);
      if (err)
	val32 = strtoul(value.c_str(),NULL,0); // convert to an unsigned long
      if (carbonIsForcible(mHandle,mNet))
	carbonForce(mHandle,mNet,&val32);
      else carbonDepositFast(mHandle,mNet,&val32,NULL);
      return true;
    }
    if (mNumWords == 2)
    {
      CarbonUInt64 val64 = strtoull(value.c_str(),NULL,0); // convert to an unsigned long long
      if (carbonIsForcible(mHandle,mNet))
	carbonForce(mHandle,mNet,(CarbonUInt32*) &val64);      
      else carbonDepositFast(mHandle,mNet,(CarbonUInt32*) &val64,NULL);
      return true;
    }
    std::string msg = "Parameter value greater then 64 bits: " + name();
    SC_REPORT_WARNING(ARM_CM_SC_MSG_ID,msg.c_str());    
    return false;    
  }
};

#endif
