/// \file        icm_scx_memory_ports_callback.h
/// \brief       This file has a C-callback implementation, which receives memory-port read/write calls 
/// from ICM model, and forwards to icm_scx_mem_port_if object. 
/// it assumes that the context received is "icm_scx_mem_port_context_wrapper" type, defined in
/// icm_scx_memory_ports_typedefs.h
///
/// \copyright   Copyright 2017 ARM Limited. All rights reserved.
///
/// \note This file is not part of a stable public API and may change without notice.
/// Please refer to your legal agreement with ARM Limited to
/// understand what you are licensed to do with this code.
#ifndef icm_scx_memory_ports_callback_h
#define icm_scx_memory_ports_callback_h

#include <scx/scx.h>

#include "icm/api/icm_error.h"
#include "icm/api/icm_types.h"

#include "icm_scx_memory_ports_typedefs.h"
using namespace icm_scx;
inline int c_mem_port_debug_read(icm_model_t model, void *client_context, uint64_t addr, uint32_t num_bytes,
                                  uint32_t *bytes_processed, uint8_t *buffer)
{
  icm_scx_mem_port_context_wrapper *port_context = static_cast<icm_scx_mem_port_context_wrapper *>(client_context);
  if (!port_context)
  {
    SCX_REPORT_WARNING("c_mem_port_debug_read","called with port_context as null");
    return -1;
  }
  if (port_context->mem_port_impl == nullptr)
  {
    SCX_REPORT_WARNING("c_mem_port_debug_read","called with an invalid pointer to icm_scx_mem_port_if");
    return -1;
  }
  return port_context->mem_port_impl->mem_port_debug_read(port_context->port_index, addr, num_bytes, bytes_processed, buffer);
}
/// signature of a function used by the model to send out a write command
inline int c_mem_port_debug_write(icm_model_t model, void *client_context, uint64_t addr, uint32_t num_bytes,
                                   uint32_t *bytes_processed, const uint8_t *buffer)
{
  icm_scx_mem_port_context_wrapper *port_context = static_cast<icm_scx_mem_port_context_wrapper *>(client_context);
  if (!port_context)
  {
    SCX_REPORT_WARNING("c_mem_port_debug_read","called with port_context as null");
    return -1;
  }
  if (port_context->mem_port_impl == nullptr)
  {
    SCX_REPORT_WARNING("c_mem_port_debug_write","called with an invalid pointer to icm_scx_mem_port_if");
    return -1;
  }
  return port_context->mem_port_impl->mem_port_debug_write(port_context->port_index, addr, num_bytes, bytes_processed, buffer);
}

#endif
