/// \file        icm_scx_memory_ports_typedefs.h
/// \brief       defines some data types useful in implementing ICM memory ports
/// 1. Interface to receive debug memory read/write requests
/// 2. a context wrapper, to encapsulate index# and Interface handle
///
/// \copyright   Copyright 2017 ARM Limited. All rights reserved.
///
/// \note This file is not part of a stable public API and may change without notice.
/// Please refer to your legal agreement with ARM Limited to
/// understand what you are licensed to do with this code.
#ifndef icm_scx_memory_ports_typedefs_h
#define icm_scx_memory_ports_typedefs_h

#include <iostream>

namespace icm_scx
{

// interface that a client class needs to implement, in order to receive callbacks
// for memory read/writes via ICM memory_ports apis.
class icm_scx_mem_port_if
{
public:
  icm_scx_mem_port_if() {}
  virtual ~icm_scx_mem_port_if() {}
  virtual int mem_port_debug_read(uint32_t index, uint64_t addr, uint32_t num_bytes, uint32_t *bytes_processed,
                                   uint8_t *buffer)
  {
    std::cout << "virtual dummy function mem_port_debug_read called\n";
    return -1;
  }
  virtual int mem_port_debug_write(uint32_t index, uint64_t addr, uint32_t num_bytes, uint32_t *bytes_processed,
                                    const uint8_t *buffer)
  {
    std::cout << "virtual dummy function mem_port_debug_write called\n";
    return -1;
  }
};

/// \brief a utility class to simplify multiple ports context management
/// intent: all the callbacks are set to a single Read (or write) function pointer, which will receive
/// the following context wrapper:  port# for which the call is being made AND 'pointer to the memory port implementation', 
/// instead of just 'pointer to the memory port implementation'.
///
/// Alternative would be to include 'port index' or 'port name' in each of the read/write call. and ICM model's individual 
/// ports should not have to worry about such port indexing, which is meaningful only at ICM model level
struct icm_scx_mem_port_context_wrapper
{
  uint32_t port_index;
  icm_scx_mem_port_if *mem_port_impl;
  icm_scx_mem_port_context_wrapper(uint32_t index = 0, icm_scx_mem_port_if *context = nullptr)
  {
    port_index = index;
    mem_port_impl = context;
  }
};

} // icm_scx

#endif
