//-------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited.
//
//            (C) COPYRIGHT 2015 ARM Limited.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited.
//
// Please refer to your legal agreement with ARM Limited to
// understand what you are licensed to do with this example code.
//-------------------------------------------------------------------

#ifndef ARM_MODELUTILS_BASICUART_H
#define ARM_MODELUTILS_BASICUART_H

#include <ostream>
#include <string>

#include <systemc>

#include "TLM/arm_axi4_socket.h"
#include <tlm_utils/simple_target_socket.h>

namespace ARM
{
namespace Models
{

/*
 * Class to hold characters printed to a UART to form whole printed lines
 */
class BasicUART : public sc_core::sc_module
{
    typedef ARM::AXI4::Payload                           PayloadT;
    typedef ARM::AXI4::Phase                             PhaseT;
    typedef std::pair<PayloadT*, uint64_t> TimedPayloadT;
    typedef ARM::AXI4::SimpleSlaveSocket<BasicUART>   SlaveSocketT;

protected:
    /* The accumulated line of text */
    std::string line;

    /* True if characters have been added but not printed */
    bool pending_printing;

    /* Prefix for every printed line */
    std::string prefix;

    std::ostream &stream;

public:
    SC_HAS_PROCESS(BasicUART);

    BasicUART(sc_core::sc_module_name p_scName, std::ostream &stream_, std::string prefix_, int width_=32);
    
    virtual ~BasicUART();
   
    tlm::tlm_sync_enum nb_transport_fw(PayloadT& payload, PhaseT& phase);

    // Pos/NegEdge functions
    void clock_posedge();
    void clock_negedge();

public:
    sc_core::sc_in<bool> clock;

    SlaveSocketT tSkt;
    tlm_utils::simple_target_socket<BasicUART> directTskt;

private:
    /*
     * Add a single character to the end of the line.  If this is a NL (0x0a) or EOT (0x04) , the
     * line will be BasicUART::flush-ed.  Returns true iff EOT is printed
     */
    bool add_character(char chr);

    /* Print all pending characters as a line and clear pending_printing */
    void flush();

    /* Clear the held line and reset pending_printing */
    void clear_line();

    unsigned int transport_dbg(tlm::tlm_generic_payload &trans);

private:
    std::deque <TimedPayloadT> r_queue;
    std::deque <TimedPayloadT> w_queue;
    std::deque <TimedPayloadT> wd_queue;

    PayloadT*                  r_current;
    PayloadT*                  b_current;

    bool                       r_ready_pending;
    bool                       b_ready_pending;
    
    uint64_t                   now;
};

}
}

#endif // ARM_MODELUTILS_BASICUART_H
