//-------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited.
//
//            (C) COPYRIGHT 2015 ARM Limited.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited.
//
// Please refer to your legal agreement with ARM Limited to
// understand what you are licensed to do with this example code.
//-------------------------------------------------------------------

#include "RAMBlock.h"
#include <cstring>

namespace ARM
{
namespace Models
{
  RAMBlock::RAMBlock()
  {
  }

  RAMBlock::~RAMBlock()
  {
    for (std::map<uint64_t,unsigned char*>::iterator page_it=pages.begin(); page_it!=pages.end(); ++page_it)
    {
        delete[] page_it->second;
    }

  }
  
  unsigned char* RAMBlock::get_page(uint64_t address)
  {
    address &= ~(RAM_BLOCK_PAGE_SIZE - 1);
    std::map<uint64_t,unsigned char*>::iterator page_it = pages.find(address);
    if (page_it == pages.end())
    {
        unsigned char* new_page = new unsigned char[RAM_BLOCK_PAGE_SIZE];
        for (unsigned i=0; i < RAM_BLOCK_PAGE_SIZE; i++)
            new_page[i] = 0xDF;
        pages[address] = new_page;
        return new_page;
    }
    return page_it->second;
  }

  void RAMBlock::read(uint64_t address, unsigned char* data, unsigned length)
  {
    while (length)
    {
        unsigned page_offset = address & (RAM_BLOCK_PAGE_SIZE - 1);
        unsigned in_page = RAM_BLOCK_PAGE_SIZE - page_offset;
        unsigned char* page = get_page(address);
        if (in_page > length)
            in_page = length;
        memcpy(data, &page[page_offset], in_page);
        length -= in_page;
        address += in_page;
        data += in_page;
    }
  }

  void RAMBlock::write(uint64_t address, unsigned char* data, unsigned length)
  {
    while (length)
    {
        unsigned page_offset = address & (RAM_BLOCK_PAGE_SIZE - 1);
        unsigned in_page = RAM_BLOCK_PAGE_SIZE - page_offset;
        unsigned char* page = get_page(address);
        if (in_page > length)
            in_page = length;
        memcpy(&page[page_offset], data, in_page);
        length -= in_page;
        address += in_page;
        data += in_page;
    }
  }
};
};
