//-------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited.
//
//            (C) COPYRIGHT 2015 ARM Limited.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited.
//
// Please refer to your legal agreement with ARM Limited to
// understand what you are licensed to do with this example code.
//-------------------------------------------------------------------


#ifndef ARM_MODELUTILS_SIMPLEBUS_H
#define ARM_MODELUTILS_SIMPLEBUS_H

#include "SimpleBusBase.h"

namespace ARM
{
namespace Models
{
  
#define AXI4_NUM_CHANNELS 9   // There are actually 8 channels starting at channel 1
#define AXI4_CHANNEL_MASK(p) (p & 0x0f0)
#define AXI4_CHANNEL_INDEX(c) (c >> 4)
#define AXI4_CHANNEL_READY(c) ((ARM::AXI4::Phase) (channel + 1))

class SimpleBus : public SimpleBusBase
{
    typedef std::tuple<PayloadT*, PhaseT,uint64_t> PayloadPhaseTimeT;
    typedef ARM::AXI4::SimpleMasterSocket<SimpleBus>  MasterSocketT;
    typedef ARM::AXI4::SimpleSlaveSocket<SimpleBus>   SlaveSocketT;
    tlm::tlm_sync_enum nb_transport_fw(PayloadT& payload, PhaseT& phase);
    tlm::tlm_sync_enum nb_transport_bw(PayloadT& payload, PhaseT& phase);

public:
    SC_HAS_PROCESS(SimpleBus);
    //Constructor
    SimpleBus(sc_core::sc_module_name p_scName, uint32_t numMasters, uint32_t numTargets, uint32_t busWidthBits_ = 128);

    //Destructor
    ~SimpleBus();
    
    // master port 
    MasterSocketT **iSkt;

    //target port
    SlaveSocketT tSkt;

    // DSGCM-8467 - back pressure queues
    void negedge();
    bool seen_channel(int channel);
    void save_payload(PayloadT& payload, PhaseT& phase,int channel,int channel_index);
    void send_payload(int channel_index);
    void report_payload(const char* prefix,PayloadT& payload, PhaseT& phase);
    uint64_t prev_time;
    std::deque <PayloadPhaseTimeT> bw_queues[AXI4_NUM_CHANNELS];
    std::vector<PhaseT> prev_phases;
};
 
}
}

#endif // ARM_MODELUTILS_SIMPLEBUS_H
