//-------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited.
//
//            (C) COPYRIGHT 2015 ARM Limited.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited.
//
// Please refer to your legal agreement with ARM Limited to
// understand what you are licensed to do with this example code.
//-------------------------------------------------------------------
#ifndef ARM_MODELUTILS_SIMPLEBUSBASE_H
#define ARM_MODELUTILS_SIMPLEBUSBASE_H

#include <systemc>

#include "TLM/arm_axi4.h"
#include "TargetMemoryMap.h"

namespace ARM
{
namespace Models
{

class SimpleBusBase : public sc_core::sc_module
{
public:
    typedef ARM::AXI4::Payload                           PayloadT;
    typedef ARM::AXI4::Phase                             PhaseT;
    
    SC_HAS_PROCESS(SimpleBusBase);
    //Constructor
    SimpleBusBase(sc_core::sc_module_name p_scName, uint32_t numMasters_, uint32_t numTargets_, uint32_t busWidthBits_) :
    sc_module(p_scName),
    clock("clock_bus"),
    memoryMap(numTargets_),
    numMasters(numMasters_),
    numTargets(numTargets_)
    {
    }
    
    //Destructor
    ~SimpleBusBase()=default;

    void addMap(uint32_t portId, uint64_t baseAddr, uint32_t slaveSize)
    {
      memoryMap.addMap(portId, baseAddr, slaveSize);
    }
    
    void dump_address_map(std::ostream& os)
    {
      memoryMap.dump_address_map(os);
    }
    
    void dump_address_map(const std::string& fname)
    {
      std::ofstream ofs;
      ofs.open(fname.c_str());
      if (!ofs.is_open())
      {
	std::string msg("Error opening file ");
	msg += fname;
	SC_REPORT_WARNING(name(),msg.c_str());
      }
      else memoryMap.dump_address_map(ofs);
    }
    
    void load_address_map(std::istream& is)
    {
      memoryMap.load_address_map(is);
    }
    
    void load_address_map(const std::string& fname)
    {
      std::ifstream ifs;
      ifs.open(fname.c_str());
      if (!ifs.is_open())
      {
	std::string msg("Error opening file ");
	msg += fname;
	SC_REPORT_WARNING(name(),msg.c_str());
      }
      else memoryMap.load_address_map(ifs);      
    }
    
    //-------------- clock ----------------//
    sc_core::sc_in<bool> clock;
protected:    
    TargetMemoryMap memoryMap;
    uint32_t numMasters; 
    uint32_t numTargets;
};

}
}

#endif // ARM_MODELUTILS_SIMPLEBUSBASE_H
