//-------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited.
//
//            (C) COPYRIGHT 2015 ARM Limited.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited.
//
// Please refer to your legal agreement with ARM Limited to
// understand what you are licensed to do with this example code.
//-------------------------------------------------------------------

#ifndef ARM_MODELUTILS_SIMPLEFLASHIMP_H
#define ARM_MODELUTILS_SIMPLEFLASHIMP_H

#include <stdint.h>
#include <algorithm>
#include <list>

#include <systemc>
#include <tlm_utils/simple_target_socket.h>

#include "RAMBlock.h"
#include "TLM/arm_axi4.h"

namespace ARM
{
namespace Models
{
namespace ModelSimpleMem
{

class SimpleFlashConfig
{
public:
    RAMBlock *ram_block;
    uint32_t busWidthBits;
    unsigned debugLevel;
    unsigned read_delay;
    unsigned cache_delay;
    uint32_t cache_line_bytes;
    unsigned int cache_line_count;
    unsigned max_outstanding;

    SimpleFlashConfig()
      : ram_block(NULL)
      , busWidthBits(128)
      , debugLevel(0)
      , read_delay(0)
      , cache_delay(0)
      , cache_line_bytes(0)
      , cache_line_count(0)
      , max_outstanding(0xffffffff)
    {
    }
};

class SimpleFlashImp : public sc_core::sc_module
{
    typedef ARM::AXI4::SimpleSlaveSocket<SimpleFlashImp> SlaveSocketT;

    typedef ARM::AXI4::Phase PhaseT;

protected:
    typedef ARM::AXI4::Payload PayloadT;

public:
    SC_HAS_PROCESS(SimpleFlashImp);
    // Constructor
    SimpleFlashImp(sc_core::sc_module_name p_scName, const SimpleFlashConfig &config);

    // Destructor
    ~SimpleFlashImp();

    tlm::tlm_sync_enum nb_transport_fw(PayloadT &payload, PhaseT &phase);

    unsigned int transport_dbg(tlm::tlm_generic_payload &temp);

    // Pos/NegEdge functions
    void clock_posedge();
    void clock_negedge();

public:
    //-------------- clock ----------------//
    sc_core::sc_in<bool> clock;

    // target port
    SlaveSocketT tSkt;
    tlm_utils::simple_target_socket<SimpleFlashImp> directTskt;

    RAMBlock *ram_block;
    unsigned debugLevel;

    typedef std::pair<PayloadT *, uint64_t> TimedPayloadT;

protected:
    unsigned get_time_ready(uint64_t address);

private:
    struct CacheLine
    {
        uint64_t address;
        uint64_t last_used;
        uint64_t time_ready;

        bool is_valid() const { return last_used > 0; }
    };
    typedef std::vector<CacheLine> Cache;

    Cache::iterator find_cache_line(uint64_t address);

    typedef std::list<TimedPayloadT> TimedPayloadQueue;
    TimedPayloadQueue r_queue;

    typedef std::pair<PayloadT *, unsigned> PayloadAndBeat;
    typedef std::deque<PayloadAndBeat> IncomingQueue;
    // Used to hold requests until we can process them into the read queue (r_queue).
    IncomingQueue read_buffer;
    tlm::tlm_sync_enum check_read_buffer();

    // We still have write queues as we still need to process write requests
    // even though we ignore them.
    TimedPayloadQueue w_queue;
    TimedPayloadQueue wd_queue;

    Cache mini_cache;

    // queue a add payload. Also call ref() on actual payload, to track the reference.
    void queue_insert(TimedPayloadQueue &queue, const TimedPayloadT &payload, PayloadT& actual_axi_payload);

    PayloadT *r_current;
    PayloadT *b_current;

    bool r_ready_pending;
    bool b_ready_pending;

    const char *m_name;

    unsigned m_delay_read;
    unsigned m_delay_cache;
    unsigned m_max_outstanding;

    unsigned m_cache_line_count;
    unsigned m_cache_line_bytes;

    unsigned access_count;
    unsigned cache_hits;

    uint64_t cache_address_mask;

    uint64_t now;

    uint8_t debug_level;
    uint64_t beats_in_flash;
};
}
}
}

#endif // ARM_MODELUTILS_SIMPLEMEMIMP_H
