//-------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited.
//
//            (C) COPYRIGHT 2015 ARM Limited.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited.
//
// Please refer to your legal agreement with ARM Limited to
// understand what you are licensed to do with this example code.
//-------------------------------------------------------------------

#ifndef ARM_MODELUTILS_SIMPLEMEM_H
#define ARM_MODELUTILS_SIMPLEMEM_H

#include <systemc>

#include "RAMBlock.h"
#include "TLM/arm_axi4_socket.h"

namespace ARM
{
namespace Models
{


class SimpleMemConfig
{
public:
    RAMBlock* ram_block;
    uint32_t busWidthBits;
    unsigned int debugLevel;
    unsigned int delay;
    bool read_only;

    const char *memory_dump_file;
    uint64_t dump_min_address;
    uint64_t dump_max_address;

    SimpleMemConfig()
      : ram_block(NULL)
      , busWidthBits(128)
      , debugLevel(0)
      , delay(0)
      , read_only(false)
      , memory_dump_file(0)
      , dump_min_address(0)
      , dump_max_address(0xffff)
    {
    }
};

namespace ModelSimpleMem { class SimpleMemImp; }

class SimpleMem : public sc_core::sc_module
{
public:
    SimpleMem(sc_core::sc_module_name p_scName, SimpleMemConfig config);

    ~SimpleMem();

    void dump_memory();

    sc_core::sc_in<bool> clock;

    ARM::TLM::BaseSlaveSocket<ARM::AXI4::ProtocolType> tSkt;

    tlm::tlm_target_socket<> directTskt;
private:
    ModelSimpleMem::SimpleMemImp* module;
};

}
}

#endif // ARM_MODELUTILS_SIMPLEMEM_H

