//-------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited.
//
//            (C) COPYRIGHT 2015 ARM Limited.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited.
//
// Please refer to your legal agreement with ARM Limited to
// understand what you are licensed to do with this example code.
//-------------------------------------------------------------------

#ifndef ARM_MODELUTILS_SIMPLEMEMIMP_H
#define ARM_MODELUTILS_SIMPLEMEMIMP_H

#include <stdint.h>
#include <algorithm>
#include <signal.h>

#include <systemc>
#include <tlm_utils/simple_target_socket.h>

#include "RAMBlock.h"
#include "SimpleMem.h"
#include "TLM/arm_axi4.h"

namespace ARM
{
namespace Models
{
namespace ModelSimpleMem
{

class SimpleMemImp : public sc_core::sc_module
{
    typedef ARM::AXI4::SimpleSlaveSocket<SimpleMemImp>   SlaveSocketT;

    typedef ARM::AXI4::Payload                           PayloadT;
    typedef ARM::AXI4::Phase                             PhaseT;

public:
    SC_HAS_PROCESS(SimpleMemImp);
    //Constructor
    SimpleMemImp(sc_core::sc_module_name p_scName, const SimpleMemConfig &config);

    //Destructor
    ~SimpleMemImp();

    tlm::tlm_sync_enum nb_transport_fw(PayloadT& payload, PhaseT& phase);

    unsigned int transport_dbg(tlm::tlm_generic_payload & temp);

    // Pos/NegEdge functions
    void clock_posedge();
    void clock_negedge();

    void dump_memory();
public:

    //-------------- clock ----------------//
    sc_core::sc_in<bool> clock;

    //target port
    SlaveSocketT               tSkt;
    tlm_utils::simple_target_socket<SimpleMemImp> directTskt;

    RAMBlock*                  ram_block;
    unsigned                   debugLevel;

    typedef std::pair<PayloadT*, uint64_t> TimedPayloadT;

private:
    std::deque <TimedPayloadT> r_queue;
    std::deque <TimedPayloadT> w_queue;
    std::deque <TimedPayloadT> wd_queue;

    PayloadT*                  r_current;
    PayloadT*                  b_current;

    uint32_t                   r_current_delay;
    uint32_t                   w_current_delay;

    bool                       r_ready_pending;
    bool                       b_ready_pending;

    const char*                m_name;

    uint32_t                   m_delay_read;
    uint32_t                   m_delay_write;

    bool                       m_readonly;

    uint64_t                   now;

    //#{
    /// Memory dump handling.
    static void sigint_handler(int signum);
    static void install_signal_handler();
    static bool installed_sigint_handler;
    static struct sigaction previous_signal_handler;

    const char *m_memory_dump_file;
    uint64_t m_memory_dump_min;
    uint64_t m_memory_dump_max;

    /**
     * List of SimpleMemImp objects
     * Used for dumping memory on exit (through SIGINT handler).
     **/
    static std::vector<SimpleMemImp *> mem_imps;
    //#}
};

}
}
}

#endif // ARM_MODELUTILS_SIMPLEMEMIMP_H
