//-------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited.
//
//            (C) COPYRIGHT 2015 ARM Limited.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited.
//
// Please refer to your legal agreement with ARM Limited to
// understand what you are licensed to do with this example code.
//-------------------------------------------------------------------


#ifndef TARGET_MEMORY_MAP
#define TARGET_MEMORY_MAP

#include <map>

struct MemoryMap{
    uint64_t baseAddress; 
    uint32_t slaveSize;

    MemoryMap() : baseAddress(0), slaveSize(0) {}
 
    MemoryMap(uint64_t addr, uint32_t sz) : baseAddress(addr), slaveSize(sz) {}
};

class TargetMemoryMap
{
public:
    static const uint32_t INVALID_PORT = 0xFFFFFFFF;

    TargetMemoryMap(uint32_t numTargets_) : numTargets(numTargets_) { }
   
    void addMap(uint32_t portNum, uint64_t address, uint32_t size)
    {
        MemoryMap map(address, size);
        mappings[portNum] = map;

        assert(mappings.size() <= numTargets);
    }

    uint32_t decodeAddress(uint64_t address)
    {
        for(MappingsMap::iterator it = mappings.begin() ; it != mappings.end(); ++it)
        {
            MemoryMap &map = it->second;
            uint64_t endAddress = map.baseAddress + map.slaveSize;

            if(address >= map.baseAddress && address < endAddress)
            {
                return it->first;
            }
        }
        return INVALID_PORT;
    }

    void reset()
    {
        mappings.clear();
    }

    void load_address_map(std::istream& is)
    {
      reset();
      std::string line;
      while(!is.eof())
      {
	std::getline(is,line);
	if (line[0] == '#')  // ignore comment
	  continue;

	int id;
	uint64_t baseAddress,slaveSize;
	std::stringstream ss(line);
	ss >> std::dec >> id;
	ss >> std::hex >> baseAddress;
	ss >> std::hex >> slaveSize;
	addMap(id,baseAddress,slaveSize);
      }
    }

    void dump_address_map(std::ostream& os)
    {
      for (auto it:mappings)
      {
	os << std::dec << it.first << " ";
	os << std::hex << std::showbase << it.second.baseAddress << " " << it.second.slaveSize << std::endl;
      }
    }
    
private:
    typedef std::map<uint32_t, MemoryMap> MappingsMap;
    MappingsMap mappings;
    uint32_t numTargets;
};

#endif

