//
// (C) COPYRIGHT 2017 ARM Limited. ALL RIGHTS RESERVED.
//

#ifndef ARM_TRAFFICGENERATOR_H
#define ARM_TRAFFICGENERATOR_H

#include <deque>
#include <map>

#include "arm_axi4.h"

using namespace ARM::AXI4;

class TrafficGenerator : public sc_core::sc_module
{
protected:
    SC_HAS_PROCESS(TrafficGenerator);

    enum ChannelState
    {
        CLEAR,
        REQ,
        ACK
    };

    /* Outgoing queues per channel. */
    std::deque<Payload*> aw_queue;
    std::deque<Payload*> w_queue;
    std::deque<Payload*> ar_queue;
    std::deque<Payload*> cr_queue;
    std::deque<Payload*> wack_queue;
    std::deque<Payload*> rack_queue;

    /* Current state of each fw channel. */
    ChannelState aw_state;
    ChannelState w_state;
    ChannelState ar_state;
    ChannelState cr_state;


    /* Incoming communications to push at posedge. */
    Payload* cr_incoming;
    Payload* wack_incoming;
    Payload* rack_incoming;

    /* Number of beats (of the front Payload of w_queue) already sent. */
    unsigned w_beat_count;

    typedef std::map<std::string, Command> CommandMap;
    typedef std::map<std::string, Size> SizeMap;
    typedef std::map<std::string, Burst> BurstMap;
    static std::map<std::string, Command> cmd_case;
    static std::map<std::string, Size> sz_case;
    static std::map<std::string, Burst> burst_case;

    void clock_posedge();
    void clock_negedge();

    tlm::tlm_sync_enum nb_transport_bw(Payload& payload,
        Phase& phase);

public:
    TrafficGenerator(sc_core::sc_module_name name,unsigned port_width=128,ARM::TLM::Protocol protocol=ARM::TLM::PROTOCOL_ACE);

    /* Add a payload to the traffic queue. */
    void add_payload(Command command, uint64_t address,
        Size size, uint8_t len, Burst burst =
        BURST_INCR);

    /* Extract transactions from input m3i file */
    void read_file(const char* infile);

    SimpleMasterSocket<TrafficGenerator> master;

    sc_core::sc_in<bool> clock;

private:
    void parse_payload(std::string const&);

};


#endif // ARM_TRAFFICGENERATOR_H
