#!/bin/tcsh 
## this file must be sourced, not executed.
## it sets up your environment for using Arm Cycle Model Studio


if ( $?0 ) then
    echo $0 | egrep 'setup.csh' > /dev/null
    if ( $status == 0 ) then
        echo ""
        echo "ERROR: Modesl Studio's setup.csh this must be SOURCED, NOT EXECUTED in a shell."
        echo "Try: source setup.csh"
        echo ""
        exit 1
    endif
endif

# LSOF is used so that the script can determine it's own location
if ( -x /usr/sbin/lsof ) then 
  set LSOF=/usr/sbin/lsof
else if ( -x /usr/bin/lsof ) then
  set LSOF=/usr/bin/lsof
else
  echo "ERROR could not find location of lsof command, needed for determining the source location for the setup.csh file"
  exit 1
endif

# LSOF returns the list of open files.  If you have a script called setup.csh
# that calls the CMS setup.csh, LSOF will return the path to both scripts.
# Using tail to get the last one opened (this script)
set SCRIPTNAME=`$LSOF -w +p $$ | grep -oE /.\*setup.csh | tail -1`
pushd `dirname $SCRIPTNAME`/.. > /dev/null
set INSTALLPATH=`pwd`
popd > /dev/null

##
setenv CARBON_HOME $INSTALLPATH
# do not unnecessarily extend variables with duplicates
if( !("$PATH" =~ *"$CARBON_HOME/bin"*) )   setenv PATH $CARBON_HOME/bin:$PATH

if( $?LD_LIBRARY_PATH ) then 
    if( !($LD_LIBRARY_PATH =~ *"$CARBON_HOME/Linux64/lib"*) ) then
	setenv LD_LIBRARY_PATH $CARBON_HOME/Linux64/lib:$LD_LIBRARY_PATH
    endif
else
    setenv  LD_LIBRARY_PATH $CARBON_HOME/Linux64/lib
endif

## using 'gcc/' here with the plan that if gcc versions is changed the shared/libcarbon5 and related files will still continue to work
if( !($LD_LIBRARY_PATH =~ *"$CARBON_HOME/Linux64/lib/gcc/shared"*) ) then
    setenv  LD_LIBRARY_PATH $CARBON_HOME/Linux64/lib/gcc/shared:$LD_LIBRARY_PATH
endif






