# -*- Makefile -*-
#
# *****************************************************************************
# Copyright (c) 2003-2010,2012-2013,2015-2019 by Arm Limited (or its affiliates). All Rights Reserved.
# *****************************************************************************
#
# compiler-dependent makefile fragment

# Define where compiler can be found
ifndef CARBON_GCC7_VERSION
  CARBON_GCC7_VERSION := 73
endif
CARBON_COMPILER_DIST:=${CARBON_HOME}/${CARBON_TARGET_ARCH}/gcc${CARBON_GCC7_VERSION}


# Where gcc wants to look for libraries
CARBON_GCC_LIB_Host := lib64
CARBON_GCC_LIB_Target := lib64

CARBON_COMPILER_LIB_DIRS := $(CARBON_COMPILER_DIST)/$(CARBON_GCC_LIB_Target)

# C++ runtime libraries needed to export a stand-alone .a file
#

CARBON_COMPILER_SUPPORTS_PCH:=1

# We could potentially need to run gcc compiled for the target architecture
# and speedcc compiled for the host architecture, so we'd need both libraries.
CARBON_LIBRARY_PATH := $(CARBON_HOME)/$(CARBON_HOST_ARCH)/gcc/$(CARBON_GCC_LIB_Host)
ifneq ($(CARBON_HOST_ARCH),$(CARBON_TARGET_ARCH))
# If host doesn't match target, append target libraries.
# (Can't use += because that adds a space.)
  CARBON_LIBRARY_PATH := $(CARBON_LIBRARY_PATH):$(CARBON_COMPILER_LIB_DIRS)
endif

# Use "env -i" to prevent user's environment from causing problems.
# We set COMPILER_PATH to find our copies of tools like `as' and `ld'.
CARBON_COMPILER_ENV = env -i LANG=C \
    COMPILER_PATH=${CARBON_HOME}/${CARBON_TARGET_ARCH}/bin \
    HOME=$(HOME) LD_LIBRARY_PATH=$(CARBON_LIBRARY_PATH)


CARBON_CBUILD_CC  = $(CARBON_COMPILER_DIST)/bin/gcc
CARBON_CBUILD_CXX = $(CARBON_COMPILER_DIST)/bin/g++
CARBON_CBUILD_LD  = $(CARBON_CBUILD_CXX)

# Indicate we're gcc 4 or better
CARBON_GCC4=1
CARBON_GCC7=1

##
# Search all passed in options for -O<level> flags
CARBON_CXX_OPTLIST:=$(filter -O%,$(CXX_EXTRA_FLAGS) $(CXX_WC_FLAGS))

# Find the last one specified
ifeq (,$(CARBON_CXX_OPTLIST))
CARBON_CXX_OPT := -O3 #-O2
else
CARBON_CXX_OPT:=$(word $(words $(CARBON_CXX_OPTLIST)),$(CARBON_CXX_OPTLIST))
endif

ifneq (,$(findstring -pg,$(CXX_EXTRA_FLAGS) $(CXX_WC_FLAGS)))
CARBON_PROFILING=1
endif



CARBON_OPTFLAGS = --param large-function-growth=250 -ftree-vectorize

ifneq (,$(findstring $(CARBON_CXX_OPT),-O2 -O3))
  ifeq ($(CARBON_TARGET_ARCH),Linux)

    CARBON_OPTFLAGS += -funroll-loops -minline-all-stringops
  endif

  ifeq ($(CARBON_TARGET_ARCH),Linux64)
    CARBON_OPTFLAGS += -funroll-loops
  endif

  # options useful when you want to profile with gprof
  ifeq ($(CARBON_PROFILING),1)
    CARBON_OPTFLAGS += -finline-limit=2000
  else
    # 4.2 snapshot 20060715 crashes with -ftracer on a neteffect test.
    CARBON_OPTFLAGS += -fvisibility-inlines-hidden \
	        -finline-functions -finline-limit=2000

  endif
else				# -Os or -O1 or -O0
  ifneq (,$(findstring $(CARBON_CXX_OPT),-Os -O1))
    ifeq ($(CARBON_TARGET_ARCH),Linux)
      CARBON_OPTFLAGS += -march=pentium4 -msse -minline-all-stringops
    endif
  else				# -O0
  endif
endif


# Basic compiler-dependent flags
#
CARBON_CXX_FLAGS_Linux =
# We used to have '-m64' in 'CARBON_CXX_FLAGS_Linux64', but it's the default
# when gcc is running on x86_64 and it's not allowed when targetting AArch64.
CARBON_CXX_FLAGS_Linux64 = -D__STDC_CONSTANT_MACROS -D__STDC_LIMIT_MACROS

# Define magic values that turn C++ exceptions off/on
CARBON_EXCEPTIONS_OFF:=-fno-exceptions
CARBON_EXCEPTIONS_ON :=-fexceptions

CARBON_CXX_FLAGS = $(CARBON_EXCEPTIONS_OFF) $(CARBON_CXX_FLAGS_$(CARBON_TARGET_ARCH))
CARBON_CXX_DEF_COMPILE_FLAGS= $(CARBON_CXX_FLAGS)

# Option to disable showing source info in diagnostics (TODO: consider adding '-fno-show-column')
CARBON_DIAGS_OMIT_SRC = -fno-diagnostics-show-caret

ifeq ($(CARBON_PROFILING),1)
else
ifeq ($(GCC_NO_WALL),)

# strict-aliasing warnings triggered by our static bitvector constants...
# uninitialized triggered by DW protected uninitialized struct.
# parentheses warnings are to clarify operator precedence where it's error-prone - not a functional error

# FIXME64 - tried pulling some flags out of SPEEDCC to get through the back end...
# TODO: Make-winx has '-Wall -Wextra ... -Werror'; Linux and Windows should be kept in sync (DSGCM-5387)
# '-Wno-shift-negative-value' disables the '-Wextra' warning that warns about,
#   e.g. '-1 << n' -- while technically undefined in C99/C++11 and later, all common
#   hardware implements signed left shift the same as unsigned left shift (so we suppress
#   this hypothetical portability warning).
# '-Wno-int-in-bool-context' disables the '-Wextra' warning that warns about,
#   e.g. if (e ? 3 : 0) ...' -- this is not useful because the int in bool semantics
#    are the same in Verilog/SystemVerilog.  We could change codegen to wrap any
#    potentially-non-bool expression in bool contexts with '(e) != 0' but that doesn't seem
#    worthwhile, so we'll just add '-Wno-int-in-bool-context'.
#CARBON_CXX_SPEEDCC_COMPILE_FLAGS += -Wall -Wextra -Wno-shift-negative-value -Wno-int-in-bool-context -Werror -Wno-strict-aliasing -Wno-uninitialized -Wno-parentheses
# TODO: We use '-fPIC' so that we produce code that is usable in shared libraries.
# (originally added in ef550df8 by Matt Grasse)  Is there any important non-fPIC use (e.g. -testdriver)?
# non-fPIC is faster.
# FIXME: We should be adding these to CARBON_CXX_FLAGS *not* CARBON_CXX_SPEEDCC_COMPILE_FLAGS so that they get used for -noSpeedCC, too
# '-Wno-misleading-indentation' is because SpeedCC destroys indentation.
CARBON_CXX_SPEEDCC_COMPILE_FLAGS += -Wno-strict-aliasing -Wno-uninitialized -Wno-parentheses -fPIC
endif
endif

CARBON_DISABLE_DYNAMIC_CAST = -fno-rtti
CARBON_ENABLE_DYNAMIC_CAST = -frtti

CARBON_CXX_SPEEDCC_COMPILE_FLAGS += -pipe $(CARBON_DISABLE_DYNAMIC_CAST) $(CARBON_DIAGS_OMIT_SRC) -Wno-misleading-indentation $(CXX_WC_FLAGS)

# gcc4 produces bad code in some structure-intensive corner cases if
# we don't use -fno-strict-aliasing.
# <deleted customer test name> fails without -fno-tree-vrp
# TODO: check if these are needed for gcc7 and if they hurt performance
CARBON_CXX_SPEEDCC_COMPILE_FLAGS += -fno-strict-aliasing -fno-tree-vrp

# flags needed to compile shared libraries
CARBON_SHARED_COMPILE_FLAGS= -shared -fPIC
# flags needed to link shared lib.  Note that 'LIBPATH' is defined in Makefile.mc
CARBON_SHARED_LINK_FLAGS=-Wl,-rpath,$(@D)

ifeq (,$(findstring $(CARBON_COMPILER_DIST),$(LD_LIBRARY_PATH)))
  LD_LIBRARY_PATH := $(LD_LIBRARY_PATH):$(CARBON_COMPILER_LIB_DIRS)
  export LD_LIBRARY_PATH
endif

CARBON_COMPILER_LIB_LIST:=
