# -*- Makefile -*-
# *****************************************************************************
#  Copyright (c) 2002-2008,2012-2017 by Arm Limited (or its affiliates). All Rights Reserved.
# *****************************************************************************

include $(CARBON_HOME)/makefiles/Makefile.common

SEARCHPATH= $(addprefix -L,$(CARBON_LIB_DIRS))
CARBON_LIBMODE_ON= -Wl,-Bstatic
CARBON_LIBMODE_OFF= -Wl,-Bdynamic

ifneq ($(CARBON_BIN:product=),)	# If set to other than product
# If no CARBON_BIN given, the modshell libraries are in the carbon standard path
CARBON_MODSHELL_PATH = \
	$(CARBON_HOME)/obj/$(CARBON_TARGET)/modshell \
	$(CARBON_HOME)/obj/$(CARBON_TARGET)/waveform \
	$(CARBON_HOME)/obj/$(CARBON_TARGET)/wavetestbench
SEARCHPATH += $(addprefix -L,$(CARBON_MODSHELL_PATH))
endif


all: $(EXECNAME)

ifdef USER_CB_SRC
USER_CB_OBJ = $(basename $(USER_CB_SRC)).o

$(USER_CB_OBJ): $(USER_CB_SRC)
	$(CARBON_COMPILE) -o $@ $<
endif

ifdef FSDB_READ_ON
  CARBON_FSDBREAD_LIB = -lfsdbwaveform
  ifneq ($(CARBON_TARGET_ARCH),Win)
    CARBON_FSDBREAD_LIB += -lfsdbreaderconduit
  endif

  ifeq ($(CARBON_TARGET_ARCH),Linux64)
    CARBON_FSDBREAD_LIB +=  -L$(CARBON_HOME)/$(CARBON_TARGET_ARCH)/lib
    NOVAS_FSDBREAD_LIB += -lnffr -lnsys
  endif
  ifeq ($(CARBON_TARGET_ARCH),Win)
    # TODO: Windows FSDB Support
    ##CARBON_FSDBREAD_LIB += -lfsdbreaderconduit_msvc
    ##NOVAS_FSDBREAD_LIB += $(CARBON_HOME)/Win/lib/nffr.lib
  endif
endif

CARBON_DESIGN_LIBRARY = -L. -l$(MODELNAME)
ifeq ($(CARBON_TARGET_ARCH),Win)
CWTB_HAS_DLL = $(wildcard lib$(MODELNAME).dll)
ifneq ($(CWTB_HAS_DLL),)
# Windows creates a memory leak in the ARM CycleModels memory manager
# when linking with a ARM CycleModels dll
CWTB_MAIN_CXXFLAGS = -DFORCE_NO_MEMLEAK_CHECK
else
# minggw does not like -ldesign with a libdesign.lib and a dynamic preference
# So, if there is a .lib just throw the file on the command line
CWTB_HAS_WINLIB = $(wildcard lib$(MODELNAME).lib)
ifneq ($(CWTB_HAS_WINLIB),)
CARBON_DESIGN_LIBRARY = ./lib$(MODELNAME).lib
else
CARBON_DESIGN_LIBRARY = -L. -l$(MODELNAME)
endif
endif
endif

MAIN_OBJ = $(notdir $(MAIN_SRC)).o
$(MAIN_OBJ) : $(MAIN_SRC)
	$(CARBON_COMPILE) $(CWTB_MAIN_CXXFLAGS) -o $@ $<

$(EXECNAME): clean $(MAIN_OBJ) MdsModelStub.o lib$(MODELNAME).h $(USER_CB_OBJ)
	$(CARBON_LINK)  \
		-o $@ $(MAIN_OBJ) MdsModelStub.o $(USER_CB_OBJ) \
		$(CARBON_DESIGN_LIBRARY)		\
		$(CARBON_SHAREPATH)$(shell pwd)		\
		$(CARBON_SHAREPATH)$(CARBON_SHARED_DIR)	\
		$(CARBON_LIBMODE_ON) 			\
			-l$(LIBNAME)			\
			-lmodshell     			\
		$(CARBON_LIBMODE_OFF) 			\
		-Wl,-Bstatic 				\
		$(CARBON_FSDBREAD_LIB) 			\
		-lwaveform 				\
		-Wl,-Bdynamic 				\
		$(NOVAS_FSDBREAD_LIB) 			\
		$(CARBON_DEBUSSY_LIB) 			\
		$(SEARCHPATH)				\
		$(USER_LINK_OBJ)			\
		$(CWTB_LINK_OPTIONS)			\
		$(CARBON_LICENSE_LIB_LIST) -lpthread	\
		-lz $(CARBON_COMPILER_EXTRA) 		\
		$(CARBON_PURESTUBS) 			\
		$(CARBON_ARCH_LIBS)			\
		-ldl					\

MdsModelStub.o: MdsModelStub.cxx
	$(CARBON_COMPILE) -o $@ MdsModelStub.cxx

clean:
	$(RM) MdsModelStub.o $(MAIN_OBJ) $(USER_CB_OBJ)

.PHONY: all
