#!/usr/bin/env csh
#
# (C) COPYRIGHT 2018 ARM Limited. ALL RIGHTS RESERVED
#
# Sources the individual package setup scripts
#

if ( $?0 ) then
    echo $0 | \egrep 'setup.csh' > /dev/null
    if ( $status == 0 ) then
        echo ""
        echo "ERROR: The setup file must be SOURCED, NOT EXECUTED in a shell."
        echo "Try: source setup.csh"
        echo ""
        exit 1
    endif
endif  

# LSOF is used so that the script can determine it's own location
if ( -x /usr/sbin/lsof ) then 
  set LSOF=/usr/sbin/lsof
else if ( -x /usr/bin/lsof ) then
  set LSOF=/usr/bin/lsof
else
  echo "ERROR could not find location of lsof command, needed for determining the source location for the setup.csh file"
  exit 1
endif

# LSOF returns the list of open files.  If you have a script called setup.csh
# that calls setup.csh, LSOF will return the path to both scripts.
# Using tail to get the last one opened (this script)
set SCRIPTDIR=`$LSOF -w +p $$ | \grep -oE /.\*setup.csh | \tail -1`
set SCRIPTDIR=`\dirname $SCRIPTDIR`
set ROOTDIR=`\dirname $SCRIPTDIR`
set ABOVE_ROOTDIR=`\dirname $ROOTDIR`
 
if( $?LD_LIBRARY_PATH ) then 
  setenv LD_LIBRARY_PATH ${ROOTDIR}/lib:$LD_LIBRARY_PATH 
else
  setenv LD_LIBRARY_PATH ${ROOTDIR}/lib 
endif
