#!/bin/bash 
#
# Copyright 2019 ARM Limited.
# All rights reserved.
#

usage() {
    echo "Usage: $0 [-a application] specifies application to run (defaults to dhrystone) " 1>&2
    echo "          [-h --help] prints help message " 1>&2
    exit 1
}

# Script to run a Fast Model simuation
mydir="$(dirname $0)"

# Compiled SystemC executable, unique to this system
SIM=$mydir/system_test

# allow the script to be called from another location using an absolute path
#mydir="$(dirname $0)"

# Location of software programs
SW=$mydir/../../../software

# Make sure executable exists
[[ -z "$SIM" || ! -f $SIM ]] && echo Error, cannot find simulation executable && exit 0

# Process command line arguments
OPTS=`getopt -o a:h --long application:,help -n 'parse-options' -- "$@"`

if [ $? != 0 ] ; then echo "Error: failed to parse options" >&2 ; exit 1 ; fi

eval set -- "$OPTS"

# default application is dhrystone
APPNAME="dhrystone"

while true; do
  case "$1" in
    -a | --application ) APPNAME="$2" ; shift 2 ;;
    -h | --help ) usage ; shift ;;
    -- ) shift; break ;;
    * ) break ;;
  esac
done

# Attempt to copy image_mem64_hi.hex adn image_mem64_lo.hex into the system executable area
[ -f $SW/$APPNAME/image_mem64_hi.hex ] && cp $SW/$APPNAME/image_mem64_hi.hex $mydir/ || { echo "Can't find application image $APPNAME/image_mem64_hi.hex. Please build the application first." ; exit 1; }
[ -f $SW/$APPNAME/image_mem64_lo.hex ] && cp $SW/$APPNAME/image_mem64_lo.hex $mydir/ || { echo "Can't find application image $APPNAME/image_mem64_lo.hex. Please build the application first." ; exit 1; }

# perform environment setup
GCC_VERSION=6.4.0
CM_PLATFORM=gcc${GCC_VERSION//./}

# Setup the Cycle Model Runtime
source $mydir/../CM_runtime/ARM/CycleModels/etc/setup.sh
export SC_SIGNAL_WRITE_CHECK=DISABLE

# the cycle model system expects image_mem64_hi.hex and image_mem64_lo.hex to be located in the same directory
# as the simulation executable
pushd $mydir > /dev/null

# add the packaged runtime library to ensure a runtime is available
export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$mydir/../CM_runtime/ARM/CycleModels/Runtime/cms/11.0.3/Linux64/lib/gcc/shared

$SIM $*

popd > /dev/null

