#!/bin/bash 
#
# Copyright 2019 ARM Limited.
# All rights reserved.
#

usage() {
    echo "Usage: $0 [-a application] specifies application to run (defaults to dhrystone) " 1>&2
    echo "          [-h --help] prints help message " 1>&2
    exit 1
}

# Script to run a Fast Model simuation
mydir="$(dirname $0)"

# Compiled SystemC executable, unique to this system
SIM=$mydir/a5.x

# allow the script to be called from another location using an absolute path
#mydir="$(dirname $0)"

# Location of software programs
SW=$mydir/../../../software

# Make sure executable exists
[[ -z "$SIM" || ! -f $SIM ]] && echo Error, cannot find simulation executable && exit 0

# Process command line arguments
OPTS=`getopt -o a:h --long application:,help -n 'parse-options' -- "$@"`

if [ $? != 0 ] ; then echo "Error: failed to parse options" >&2 ; exit 1 ; fi

eval set -- "$OPTS"

# default application is dhrystone
APPNAME="dhrystone"

while true; do
  case "$1" in
    -a | --application ) APPNAME="$2" ; shift 2 ;;
    -h | --help ) usage ; shift ;;
    -- ) shift; break ;;
    * ) break ;;
  esac
done


APP=""

[ -f $SW/$APPNAME/"$APPNAME".axf ] && APP="-a $SW/$APPNAME/"$APPNAME".axf" || { echo "Can't find application $SW/$APPNAME/"$APPNAME".axf" ; exit 1; }

$SIM $APP $*

